@@ -1,4 +1,4 @@
-#include "progressbar.h"
+#include "../main/progressbar.h"
 
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 
@@ -51,6 +51,8 @@ using casacore::ScalarColumn;
 
 using aocommon::RaDecCoord;
 
+using wsclean::ProgressBar;
+
 std::vector<MPosition> antennas;
 
 // typedef long int integer;
