@@ -224,6 +224,11 @@ class ImageBase {
   const value_type& operator[](size_t index) const { return data_[index]; }
   value_type& operator[](size_t index) { return data_[index]; }
 
+  const value_type& Value(size_t x, size_t y) const {
+    return data_[x + y * width_];
+  }
+  value_type& Value(size_t x, size_t y) { return data_[x + y * width_]; }
+
   ImageBase<NumT>& operator+=(const ImageBase<NumT>& other) {
     assert(Size() == other.Size());
     for (size_t i = 0; i != width_ * height_; ++i) data_[i] += other[i];
