@@ -685,6 +685,12 @@ class MC2x2Base {
                               Get(3) * rhs);
   }
 
+  friend MC2x2Base<ValType> operator*(ValType lhs,
+                                      const MC2x2Base<ValType>& rhs) {
+    return MC2x2Base<ValType>{rhs.Get(0) * lhs, rhs.Get(1) * lhs,
+                              rhs.Get(2) * lhs, rhs.Get(3) * lhs};
+  }
+
   /**
    * Complex scalar multiplication
    */
@@ -693,6 +699,12 @@ class MC2x2Base {
                               Get(3) * rhs};
   }
 
+  friend MC2x2Base<ValType> operator*(std::complex<ValType> lhs,
+                                      const MC2x2Base<ValType>& rhs) {
+    return MC2x2Base<ValType>{rhs.Get(0) * lhs, rhs.Get(1) * lhs,
+                              rhs.Get(2) * lhs, rhs.Get(3) * lhs};
+  }
+
   /**
    * Scalar division assignment operator
    */
