@@ -50,6 +50,35 @@ BOOST_AUTO_TEST_CASE(inequality) {
   BOOST_CHECK(Image(1, 2, 2.0) != Image(1, 2, 3.0));
 }
 
+BOOST_AUTO_TEST_CASE(index_operator) {
+  Image image(7, 3, 0.0f);
+  for (size_t i = 0; i != 21; ++i) BOOST_CHECK_EQUAL(image[i], 0.0f);
+  for (size_t i = 0; i != 21; ++i) image[i] = i + 1.5f;
+  for (size_t i = 0; i != 21; ++i) BOOST_CHECK_EQUAL(image[i], i + 1.5f);
+}
+
+BOOST_AUTO_TEST_CASE(value) {
+  Image image(7, 3, 13.37f);
+  for (size_t x = 0; x != image.Width(); ++x) {
+    for (size_t y = 0; y != image.Height(); ++y) {
+      BOOST_CHECK_EQUAL(image.Value(x, y), 13.37f);
+    }
+  }
+  for (size_t x = 0; x != image.Width(); ++x) {
+    for (size_t y = 0; y != image.Height(); ++y) {
+      image.Value(x, y) = x * 11.0f + y * 42.0 - 8.0;
+    }
+  }
+  size_t index = 0;
+  for (size_t y = 0; y != image.Height(); ++y) {
+    for (size_t x = 0; x != image.Width(); ++x) {
+      BOOST_CHECK_EQUAL(image.Value(x, y), x * 11.0f + y * 42.0 - 8.0);
+      BOOST_CHECK_EQUAL(image[index], x * 11.0f + y * 42.0 - 8.0);
+      ++index;
+    }
+  }
+}
+
 BOOST_AUTO_TEST_CASE(median_empty) {
   BOOST_CHECK_EQUAL(Image::Median(nullptr, 0), 0.0f);
 }
