@@ -160,6 +160,21 @@ BOOST_AUTO_TEST_CASE(assign_subtract_diagonal) {
   CheckClose(a, result);
 }
 
+BOOST_AUTO_TEST_CASE(scalar_multiplication) {
+  const MC2x2 m({1.0, 2.0}, {3.0, 4.0}, {5.0, 6.0}, {7.0, 8.0});
+
+  const std::complex<double> complex_scalar(5.0, -1.0);
+  const MC2x2 expected_1({7.0, 9.0}, {19.0, 17.0}, {31.0, 25.0}, {43.0, 33.0});
+  CheckClose(m * complex_scalar, expected_1);
+  CheckClose(complex_scalar * m, expected_1);
+
+  constexpr double real_scalar = -2.0;
+  const MC2x2 expected_2({-2.0, -4.0}, {-6.0, -8.0}, {-10.0, -12.0},
+                         {-14.0, -16.0});
+  CheckClose(m * real_scalar, expected_2);
+  CheckClose(real_scalar * m, expected_2);
+}
+
 BOOST_AUTO_TEST_CASE(multiply_complex_times_real_array) {
   MC2x2 a({1.0, 2.0}, {0, 0}, {0, 0}, {3.0, 4.0});
   // Flattened real 2x2 array
@@ -220,6 +235,36 @@ BOOST_AUTO_TEST_CASE(hermitian_square) {
   BOOST_CHECK(a.HermitianSquare() == a.HermTranspose() * a);
 }
 
+BOOST_AUTO_TEST_CASE(real_hermitian_square_float) {
+  MC2x2F a1({1.0, 2.0}, {0.0, 0.0}, {0, 0}, {30.0, 31.0});
+  MC2x2FDiag a2({1.0, 2.0}, {30.0, 31.0});
+
+  MC2x2F a1_herm_square = a1.HermitianSquare();
+  std::array<float, 2> a2_real_herm_square = a2.RealHermitianSquare();
+
+  BOOST_CHECK_CLOSE(std::real(a1_herm_square.Get(0)), a2_real_herm_square[0],
+                    1e-12);
+  BOOST_CHECK(std::real(a1_herm_square.Get(1)) == 0.0);
+  BOOST_CHECK(std::real(a1_herm_square.Get(2)) == 0.0);
+  BOOST_CHECK_CLOSE(std::real(a1_herm_square.Get(3)), a2_real_herm_square[1],
+                    1e-12);
+}
+
+BOOST_AUTO_TEST_CASE(real_hermitian_square_double) {
+  MC2x2 a1({1.0, 2.0}, {0.0, 0.0}, {0, 0}, {30.0, 31.0});
+  MC2x2Diag a2({1.0, 2.0}, {30.0, 31.0});
+
+  MC2x2 a1_herm_square = a1.HermitianSquare();
+  std::array<double, 2> a2_real_herm_square = a2.RealHermitianSquare();
+
+  BOOST_CHECK_CLOSE(std::real(a1_herm_square.Get(0)), a2_real_herm_square[0],
+                    1e-12);
+  BOOST_CHECK(std::real(a1_herm_square.Get(1)) == 0.0);
+  BOOST_CHECK(std::real(a1_herm_square.Get(2)) == 0.0);
+  BOOST_CHECK_CLOSE(std::real(a1_herm_square.Get(3)), a2_real_herm_square[1],
+                    1e-12);
+}
+
 BOOST_AUTO_TEST_CASE(eigenvalue1) {
   double unit[4] = {1.0, 0.0, 0.0, 1.0};
   double e1, e2;
