@@ -405,9 +405,8 @@ class FailingAllocator {
   FailingAllocator() {}
   FailingAllocator(bool fail) { _failAllocation = fail; }
   void SetFailAllocation(bool fail) { _failAllocation = fail; }
-  bool operator==(const FailingAllocator<Tp>&) { return true; }
-  pointer allocate(size_type n,
-                   std::allocator<void>::const_pointer hint = nullptr) {
+  bool operator==(const FailingAllocator<Tp>&) const { return true; }
+  pointer allocate(size_type n, const void* hint = nullptr) {
     if (_failAllocation) throw std::bad_alloc();
     return static_cast<pointer>(malloc(n * sizeof(Tp)));
   }
@@ -510,8 +509,7 @@ class IdAllocater {
     return IdAllocater(_id + 10);
   }
   size_t Id() const { return _id; }
-  pointer allocate(size_type n,
-                   std::allocator<void>::const_pointer hint = nullptr) {
+  pointer allocate(size_type n, const void* hint = nullptr) {
     char* mem = static_cast<char*>(malloc(n * sizeof(Tp) + sizeof(_id)));
     *reinterpret_cast<size_t*>(mem) = _id;
     return reinterpret_cast<pointer>(mem + sizeof(_id));
