@@ -34,6 +34,12 @@ struct SubImage {
   bool reached_major_threshold;
 };
 
+struct ParallelDeconvolutionResult {
+  bool another_iteration_required;
+  aocommon::OptionalNumber<float> start_peak;
+  aocommon::OptionalNumber<float> end_peak;
+};
+
 class ParallelDeconvolution {
  public:
   ParallelDeconvolution(const Settings& settings);
@@ -62,23 +68,28 @@ class ParallelDeconvolution {
 
   void SetThreshold(double threshold);
 
+  void SetMinorLoopGain(double gain);
+
   bool IsInitialized() const { return !algorithms_.empty(); }
 
   /**
    * Set the multiscale auto-masking mode. This method requires that the class
    * is initialized with the multiscale algorithm.
    */
-  void SetAutoMaskMode(bool track_per_scale_masks, bool use_per_scale_masks);
+  void SetMultiscaleAutoMaskMode(bool track_per_scale_masks,
+                                 bool use_per_scale_masks);
+
+  void SetComponentOptimization(OptimizationAlgorithm algorithm);
 
   void SetCleanMask(const bool* mask);
 
   void SetSpectrallyForcedImages(std::vector<aocommon::Image>&& images);
 
   /** @param psf_images @see @ref ImageSet::LoadAndAveragePsfs. */
-  void ExecuteMajorIteration(
+  ParallelDeconvolutionResult ExecuteMajorIteration(
       ImageSet& data_image, ImageSet& model_image,
       const std::vector<std::vector<aocommon::Image>>& psf_images,
-      const std::vector<PsfOffset>& psf_offsets, bool& reached_major_threshold);
+      const std::vector<PsfOffset>& psf_offsets, double major_loop_gain);
 
   void FreeDeconvolutionAlgorithms() {
     algorithms_.clear();
@@ -87,15 +98,15 @@ class ParallelDeconvolution {
 
  private:
   /** @param psf_images @see @ref ImageSet::LoadAndAveragePsfs. */
-  void ExecuteParallelRun(
+  ParallelDeconvolutionResult ExecuteParallelRun(
       ImageSet& data_image, ImageSet& model_image,
       const std::vector<std::vector<aocommon::Image>>& psf_images,
-      const std::vector<PsfOffset>& psf_offsets, bool& reached_major_threshold);
+      const std::vector<PsfOffset>& psf_offsets, double major_loop_gain);
 
-  void ExecuteSingleThreadedRun(
+  ParallelDeconvolutionResult ExecuteSingleThreadedRun(
       ImageSet& data_image, ImageSet& model_image,
       const std::vector<std::vector<aocommon::Image>>& psf_images,
-      const std::vector<PsfOffset>& psf_offsets, bool& reached_major_threshold);
+      const std::vector<PsfOffset>& psf_offsets, double major_loop_gain);
 
   /** @param psf_images @see @ref ImageSet::LoadAndAveragePsfs. */
   void RunSubImage(SubImage& sub_image, ImageSet& data_image,
