@@ -24,6 +24,7 @@
 #include "algorithms/simple_clean.h"
 
 #include "image_set.h"
+#include "math/gain_calculations.h"
 #include "math/rms_image.h"
 #include "utils/casa_mask_reader.h"
 #include "utils/load_image_accessor.h"
@@ -85,15 +86,12 @@ Radler::Radler(const Settings& settings, const aocommon::Image& psf_image,
 
 Radler::Radler(const Settings& settings, double beam_size)
     : settings_(settings),
-      table_(),
       parallel_deconvolution_(
           std::make_unique<algorithms::ParallelDeconvolution>(settings_)),
-      auto_mask_is_finished_(false),
       image_width_(settings_.trimmed_image_width),
       image_height_(settings_.trimmed_image_height),
       pixel_scale_x_(settings_.pixel_scale.x),
       pixel_scale_y_(settings_.pixel_scale.y),
-      auto_mask_(),
       beam_size_(beam_size) {
   if (settings.spectral_fitting.mode ==
           schaapcommon::fitters::SpectralFittingMode::kForcedTerms &&
@@ -130,7 +128,31 @@ const algorithms::DeconvolutionAlgorithm& Radler::MaxScaleCountAlgorithm()
   return parallel_deconvolution_->MaxScaleCountAlgorithm();
 }
 
-void Radler::Perform(bool& reached_major_threshold,
+void Radler::SetAutoMaskMode(ImageSet& model_set, bool use_mask) {
+  if (settings_.algorithm_type == AlgorithmType::kMultiscale) {
+    parallel_deconvolution_->SetMultiscaleAutoMaskMode(!use_mask, use_mask);
+  } else if (use_mask) {
+    if (auto_mask_.empty()) {
+      // Generate the auto-mask from the model image(s)
+      auto_mask_.assign(image_width_ * image_height_, false);
+      for (size_t image_index = 0; image_index != model_set.Size();
+           ++image_index) {
+        const aocommon::Image& image = model_set[image_index];
+        for (size_t i = 0; i != image_width_ * image_height_; ++i) {
+          if (std::isfinite(image[i]) && image[i] != 0.0) auto_mask_[i] = true;
+        }
+      }
+    }
+    parallel_deconvolution_->SetCleanMask(auto_mask_.data());
+  } else {
+    if (clean_mask_.empty())
+      parallel_deconvolution_->SetCleanMask(nullptr);
+    else
+      parallel_deconvolution_->SetCleanMask(clean_mask_.data());
+  }
+}
+
+void Radler::Perform(bool& another_iteration_required,
                      size_t major_iteration_number) {
   /**
    * Because functions like convolution in schaapcommon use parallelized fors,
@@ -165,16 +187,29 @@ void Radler::Perform(bool& reached_major_threshold,
   Image integrated(image_width_, image_height_);
   residual_set.GetLinearIntegrated(integrated);
   Logger::Debug << "Calculating standard deviation...\n";
-  double stddev = integrated.StdDevFromMAD();
+  const std::pair<float, float> median_and_stddev =
+      integrated.MedianAndStdDevFromMAD();
+  double stddev = median_and_stddev.second;
   Logger::Info << "Estimated standard deviation of background noise: "
-               << FluxDensity::ToNiceString(stddev) << '\n';
+               << FluxDensity::ToNiceString(median_and_stddev.second) << '\n';
   const bool auto_mask_is_enabled =
       settings_.auto_mask_sigma || settings_.absolute_auto_mask_threshold;
   if (auto_mask_is_enabled && auto_mask_is_finished_) {
+    // Once the auto-mask is used, a more aggressive gain value can be used.
+    // This decreases the number of required minor iterations a bit.
+    parallel_deconvolution_->SetMinorLoopGain(
+        std::min(1.0, settings_.minor_loop_gain * 2.0));
     // When we are in the second phase of automasking, don't use
     // the RMS background anymore
     parallel_deconvolution_->SetRmsFactorImage(Image());
+    // Component optimization is only performed after the mask is complete
+    if (settings_.component_optimization_algorithm !=
+        OptimizationAlgorithm::kClean) {
+      parallel_deconvolution_->SetComponentOptimization(
+          settings_.component_optimization_algorithm);
+    }
   } else {
+    parallel_deconvolution_->SetMinorLoopGain(settings_.minor_loop_gain);
     Image rms_image;
     if (!settings_.local_rms.image.empty()) {
       rms_image = Image(image_width_, image_height_);
@@ -206,77 +241,126 @@ void Radler::Perform(bool& reached_major_threshold,
       parallel_deconvolution_->SetRmsFactorImage(std::move(rms_image));
     }
   }
+
+  integrated.Reset();
+
+  // When using squared joins, the stddev is not appropriate as a threshold,
+  // because the whole image will have a positive mean(/median). It is probably
+  // best to unconditionally add the mean, but at this point I didn't want to
+  // change the behaviour for non-squared joins.
+  const double threshold_bias =
+      settings_.squared_joins ? median_and_stddev.first : 0.0;
+
+  // The threshold at which deconvolution is fully finished (including the
+  // automasking stage)
+  double final_threshold;
+  if (settings_.auto_threshold_sigma) {
+    final_threshold =
+        std::max(stddev * (*settings_.auto_threshold_sigma) + threshold_bias,
+                 settings_.absolute_threshold);
+  } else {
+    final_threshold = settings_.absolute_threshold;
+  }
   if (auto_mask_is_enabled && !auto_mask_is_finished_) {
-    const double combined_auto_mask_threshold =
-        std::max(stddev * settings_.auto_mask_sigma.value_or(0.0),
-                 settings_.absolute_auto_mask_threshold.value_or(0.0));
+    const double combined_auto_mask_threshold = std::max(
+        stddev * settings_.auto_mask_sigma.value_or(0.0) + threshold_bias,
+        settings_.absolute_auto_mask_threshold.value_or(0.0));
     parallel_deconvolution_->SetThreshold(
         std::max(combined_auto_mask_threshold, settings_.absolute_threshold));
   } else if (settings_.auto_threshold_sigma) {
-    parallel_deconvolution_->SetThreshold(
-        std::max(stddev * (*settings_.auto_threshold_sigma),
-                 settings_.absolute_threshold));
+    parallel_deconvolution_->SetThreshold(final_threshold);
   }
-  integrated.Reset();
 
   Logger::Debug << "Loading PSFs...\n";
   const std::vector<std::vector<aocommon::Image>> psf_images =
       residual_set.LoadAndAveragePsfs();
 
-  if (settings_.algorithm_type == AlgorithmType::kMultiscale) {
-    if (auto_mask_is_enabled) {
-      if (auto_mask_is_finished_) {
-        parallel_deconvolution_->SetAutoMaskMode(false, true);
-      } else {
-        parallel_deconvolution_->SetAutoMaskMode(true, false);
-      }
-    }
-  } else {
-    if (auto_mask_is_enabled && auto_mask_is_finished_) {
-      if (auto_mask_.empty()) {
-        // Generate the auto-mask from the model image(s)
-        auto_mask_.assign(image_width_ * image_height_, false);
-        for (size_t image_index = 0; image_index != model_set.Size();
-             ++image_index) {
-          const aocommon::Image& image = model_set[image_index];
-          for (size_t i = 0; i != image_width_ * image_height_; ++i) {
-            if (std::isfinite(image[i]) && image[i] != 0.0)
-              auto_mask_[i] = true;
-          }
-        }
-      }
-      parallel_deconvolution_->SetCleanMask(auto_mask_.data());
-    }
+  if (auto_mask_is_enabled) {
+    SetAutoMaskMode(model_set, auto_mask_is_finished_);
   }
 
-  parallel_deconvolution_->ExecuteMajorIteration(
-      residual_set, model_set, psf_images, table_->PsfOffsets(),
-      reached_major_threshold);
+  const double major_loop_gain = math::gain_calculations::CalculateBoostedGain(
+      settings_.major_loop_gain, settings_.initial_iteration_boost,
+      major_iteration_number);
+  if (major_loop_gain != settings_.major_loop_gain) {
+    Logger::Info << "Iteration " << major_iteration_number
+                 << ": boosting major loop gain to " << major_loop_gain << '\n';
+  }
 
-  if (!reached_major_threshold && auto_mask_is_enabled &&
+  const algorithms::ParallelDeconvolutionResult result =
+      parallel_deconvolution_->ExecuteMajorIteration(
+          residual_set, model_set, psf_images, table_->PsfOffsets(),
+          major_loop_gain);
+  another_iteration_required = result.another_iteration_required;
+
+  bool auto_mask_finished_now = false;
+  if (!another_iteration_required && auto_mask_is_enabled &&
       !auto_mask_is_finished_) {
     Logger::Info << "Auto-masking threshold reached; continuing next major "
                     "iteration with deeper threshold and mask.\n";
     auto_mask_is_finished_ = true;
-    reached_major_threshold = true;
+    another_iteration_required = true;
+    auto_mask_finishing_iteration = major_iteration_number;
+    auto_mask_finished_now = true;
   }
 
-  if (settings_.major_iteration_count != 0 &&
+  if (another_iteration_required && settings_.major_iteration_count != 0 &&
       major_iteration_number >= settings_.major_iteration_count) {
-    reached_major_threshold = false;
+    another_iteration_required = false;
     Logger::Info << "Maximum number of major iterations was reached: not "
                     "continuing deconvolution.\n";
   }
 
-  if (settings_.minor_iteration_count != 0 &&
+  if (another_iteration_required && auto_mask_is_finished_ &&
+      major_iteration_number - auto_mask_finishing_iteration >=
+          settings_.major_auto_mask_iteration_count) {
+    another_iteration_required = false;
+    Logger::Info << "Performed "
+                 << major_iteration_number - auto_mask_finishing_iteration
+                 << " major iterations after reaching the auto-mask threshold. "
+                    "The stopping criterion is "
+                 << settings_.major_auto_mask_iteration_count
+                 << ": not continuing deconvolution.\n";
+  }
+
+  if (another_iteration_required && settings_.minor_iteration_count != 0 &&
       parallel_deconvolution_->FirstAlgorithm().IterationNumber() >=
           settings_.minor_iteration_count) {
-    reached_major_threshold = false;
+    another_iteration_required = false;
     Logger::Info
         << "Maximum number of minor deconvolution iterations was reached: not "
            "continuing deconvolution.\n";
   }
 
+  if (settings_.major_iteration_strategy != MajorIterationStrategy::kNormal &&
+      another_iteration_required && auto_mask_is_enabled &&
+      (!auto_mask_is_finished_ || auto_mask_finished_now)) {
+    Logger::Info << "Continuing image-based deconvolution with auto-mask.\n";
+
+    double continued_loop_gain;
+    double achieved_gain;
+    std::tie(achieved_gain, continued_loop_gain) =
+        math::gain_calculations::CalculateContinuedLoopGain(
+            settings_.major_loop_gain, settings_.major_iteration_strategy,
+            result, auto_mask_finished_now);
+    if (continued_loop_gain != 0.0) {
+      if (auto_mask_finished_now) {
+        Logger::Info << "Peak was decreased by "
+                     << std::round(100.0 * achieved_gain)
+                     << "% in this iteration; using major loop gain of "
+                     << continued_loop_gain << " to finish this iteration.\n";
+        // Because the auto-mask wasn't finished in the previous iteration, the
+        // threshold is still set to the auto-mask instead of auto-threshold
+        // value.
+        parallel_deconvolution_->SetThreshold(final_threshold);
+      }
+      SetAutoMaskMode(model_set, true);
+      parallel_deconvolution_->ExecuteMajorIteration(
+          residual_set, model_set, psf_images, table_->PsfOffsets(),
+          continued_loop_gain);
+    }
+  }
+
   residual_set.AssignAndStoreResidual();
   model_set.InterpolateAndStoreModel(
       parallel_deconvolution_->FirstAlgorithm().Fitter());
