@@ -100,6 +100,55 @@ enum class MultiscaleShape {
   kGaussianShape
 };
 
+enum class OptimizationAlgorithm {
+  /**
+   * Performs the normal masked cleaning procedure when the auto-mask threshold
+   * is reached.
+   */
+  kClean,
+  /**
+   * Use a linear equation solver. This causes the equation to be solved using
+   * a matrix decomposition algorithm, like singular value decomposition. This
+   * implies that it is exact, but very slow. This optimization makes sure that
+   * the residual flux density values are zero at the place of components.
+   */
+  kLinearEquationSolver,
+  /**
+   * Iteratively performs a line search algorithm in the gradient descent
+   * direction, thereby optimizing the component values to minimize the RMS of
+   * the residual. Because the gradient and residuals can be calculated using
+   * convolutions, this algorithm is very fast and independent of the number of
+   * fitted components. When using multiscale, this requires storing the source
+   * list.
+   */
+  kGradientDescent,
+  /**
+   * Similar to gradient descent, but adds a penalty term to the component
+   * values.
+   */
+  kRegularizedGradientDescent
+};
+
+enum class MajorIterationStrategy {
+  /**
+   * Clean until the major iteration gain, then return for a prediction-gridding
+   * round.
+   */
+  kNormal,
+  /**
+   * First, clean until the major iteration gain. Then, repeat this clean step
+   * with the same major gain value, while using the auto-mask for cleaning.
+   * Once the major gain value has been reached for the second time, the
+   * algorithm returns for a prediction-gridding round. This repeats as long as
+   * the auto-mask threshold has not been reached.
+   */
+  kDual,
+  /**
+   * Like @ref kDual, but clean until the final threshold in the second step.
+   */
+  kFull
+};
+
 /// Class to collect and set (Radler) deconvolution related settings.
 struct Settings {
   /// Trimmed image width.
@@ -214,6 +263,30 @@ struct Settings {
    */
   std::optional<double> auto_mask_sigma = std::nullopt;
 
+  /**
+   * After reaching the major gain threshold in one major iteration, continue
+   * cleaning with the auto-mask. This makes auto-masking converge faster,
+   * thereby allowing slightly deeper major gain values.
+   */
+  MajorIterationStrategy major_iteration_strategy =
+      MajorIterationStrategy::kDual;
+
+  /**
+   * This option can be set to configure the agressiveness by changing the
+   * major loop gain ('mgain') during the first and second iterations. A higher
+   * gain during the first two iterations is often possible because only a few
+   * strong sources are deconvolved in the first iterations. They are also often
+   * more in the centre, so they are less affected by w-term or beam effects,
+   * and can therefore be more agressively deconvolved. Boosting with a value
+   * of 1.5 will save approximately 0.75 major iterations.
+   *
+   * In the first iteration, the mgain will be set to: 1 - (1 - mgain) ^ value.
+   * In the second iteration, 0.5 * (value-1) + 1 will be used as value. For a
+   * typical mgain value of 0.8 and a boost value of 1.2, this result in 0.85
+   * and 0.83 for the first and second major iterations, respectively.
+   */
+  double initial_iteration_boost = 1.2;
+
   /**
    * @brief Like @ref auto_mask_sigma, but instead specifies an absolute
    * level where to stop generation of the auto-mask.
@@ -243,7 +316,13 @@ struct Settings {
    * take this into account to determine the @c reached_major_threshold value
    * returned by @ref Radler::Perform().
    */
-  size_t major_iteration_count = 20;
+  size_t major_iteration_count = 12;
+
+  /**
+   * Stopping criterion on the total number of major iterations after having
+   * reached the auto-mask threshold.
+   */
+  size_t major_auto_mask_iteration_count = 2;
 
   /**
    * When set to @c false, only positive components are cleaned. This is
@@ -329,6 +408,12 @@ struct Settings {
    * If unset/empty, Radler uses: prefix_name + "-horizon-mask.fits".
    */
   std::string horizon_mask_filename;
+  /**
+   * Algorithm used to optimize the value of the found
+   * components, in order to minimize the residuals.
+   */
+  OptimizationAlgorithm component_optimization_algorithm =
+      OptimizationAlgorithm::kClean;
 
   /**
    * Settings related to cleaning relative to a local RMS value.
