@@ -10,8 +10,6 @@
 #include "h5parm.h"
 
 #include <casacore/casa/Arrays/Cube.h>
-#include <casacore/casa/Arrays/ArrayMath.h>
-#include <casacore/casa/BasicSL/String.h>
 
 namespace schaapcommon {
 namespace h5parm {
@@ -70,8 +68,6 @@ class JonesParameters {
    * (e.g. phase and amplitude). These are the parameters as they are
    * stored (e.g. TEC values).
    * \param invert (optional default=false) Invert the parameters
-   * \param sigma_mmse (optional default=0.) Minimum mean square error parameter
-   * (remnant from BBS times, leave at 0 unless you know what you're doing).
    */
   JonesParameters(const std::vector<double>& freqs,
                   const std::vector<double>& times,
@@ -79,7 +75,7 @@ class JonesParameters {
                   GainType gain_type, InterpolationType interpolation_type,
                   hsize_t direction,
                   std::vector<std::vector<std::vector<double>>>&& parm_values,
-                  bool invert = false, float sigma_mmse = 0.);
+                  bool invert = false);
 
   /**
    * Constructor for JonesParameters with given parm_values. To be used if
@@ -91,14 +87,12 @@ class JonesParameters {
    * \param correct_type Correction type of the Jones matrix
    * \param solution Solution in format [n_ants * n_pols, n_chans]
    * \param invert (optional default=false) Invert the parameters
-   * \param sigma_mmse (optional default=0.) Minimum mean square error parameter
-   * (remnant from BBS times, leave at 0 unless you know what you're doing).
    */
   JonesParameters(
       const std::vector<double>& freqs, const std::vector<double>& times,
       const std::vector<std::string>& antenna_names, GainType gain_type,
       const std::vector<std::vector<std::complex<double>>>& solution,
-      bool invert = false, float sigma_mmse = 0.);
+      bool invert = false);
 
   /**
    * Contructor for JonesParameters. JonesParameters will extract parameter
@@ -113,8 +107,6 @@ class JonesParameters {
    * \param sol_tab2 (optional default=nullptr) soltab with parameters for
    * complex values. Shapes of sol_tab and sol_tab2 can differ
    * \param invert (optional default=false) Invert the parameters
-   * \param sigma_mmse (optional default=0.) Minimum mean square error parameter
-   * (remnant from BBS times, leave at 0 unless you know what you're doing)
    * \param parm_size (optional default=0) allows to override the vector size
    * for parm_values
    * \param missing_antenna_behavior (optional default=kError) what to do with
@@ -126,8 +118,7 @@ class JonesParameters {
                   GainType gain_type, InterpolationType interpolation_type,
                   hsize_t direction, schaapcommon::h5parm::SolTab* sol_tab,
                   schaapcommon::h5parm::SolTab* sol_tab2 = nullptr,
-                  bool invert = false, float sigma_mmse = 0.,
-                  size_t parm_size = 0,
+                  bool invert = false, size_t parm_size = 0,
                   MissingAntennaBehavior missing_antenna_behavior =
                       MissingAntennaBehavior::kError);
 
@@ -214,12 +205,10 @@ class JonesParameters {
    * in MakeComplex.
    * \param parms Reference to the complex parameters that will be inverted
    * obtained via MakeComplex
-   * \param sigma_mmse (optional default=0.) Minimum mean square error parameter
-   * (remnant from BBS times, leave at 0 unless you know what you're doing).
    * \param correct_type Correction type of the Jones matrix
    */
   static void Invert(casacore::Cube<std::complex<float>>& parms,
-                     float sigma_mmse, GainType gain_type);
+                     GainType gain_type);
 
   /// Stored Jones matrices, dimensions (nparms, nantenna, ntime*nfreq),
   /// frequency varies fastest. nparms is 2 for diagonal, 4 for full jones
