@@ -3,10 +3,10 @@
 
 #include "jonesparameters.h"
 
-#include <casacore/casa/BasicSL/Complex.h>
 #include <casacore/casa/BasicSL/Constants.h>
 
 #include <cassert>
+#include <cmath>
 #include <utility>
 
 #include <aocommon/matrix2x2.h>
@@ -27,23 +27,22 @@ JonesParameters::JonesParameters(
     const std::vector<double>& freqs, const std::vector<double>& times,
     const std::vector<std::string>& antenna_names, GainType gain_type,
     InterpolationType interpolation_type, hsize_t direction,
-    std::vector<std::vector<std::vector<double>>>&& parm_values, bool invert,
-    float sigma_mmse) {
+    std::vector<std::vector<std::vector<double>>>&& parm_values, bool invert) {
   const size_t num_parms = GetNParms(gain_type);
   parms_.resize(num_parms, antenna_names.size(), freqs.size() * times.size());
   for (size_t ant = 0; ant < antenna_names.size(); ++ant) {
     MakeComplex(parm_values, ant, freqs, gain_type, invert);
   }
   if (invert) {
-    Invert(parms_, sigma_mmse, gain_type);
+    Invert(parms_, gain_type);
   }
 }
 
 JonesParameters::JonesParameters(
     const std::vector<double>& freqs, const std::vector<double>& times,
     const std::vector<std::string>& antenna_names, GainType gain_type,
-    const std::vector<std::vector<std::complex<double>>>& solution, bool invert,
-    float sigma_mmse) {
+    const std::vector<std::vector<std::complex<double>>>& solution,
+    bool invert) {
   /*
    * Solution is in format [n_ants * n_pols, n_chans]
    * Expected format of the parm_values is [n_chans, n_ants, n_pols]
@@ -62,7 +61,7 @@ JonesParameters::JonesParameters(
   }
 
   if (invert) {
-    Invert(parms_, sigma_mmse, gain_type);
+    Invert(parms_, gain_type);
   }
 }
 
@@ -71,8 +70,8 @@ JonesParameters::JonesParameters(
     const std::vector<std::string>& antenna_names, GainType gain_type,
     InterpolationType interpolation_type, hsize_t direction,
     schaapcommon::h5parm::SolTab* sol_tab,
-    schaapcommon::h5parm::SolTab* sol_tab2, bool invert, float sigma_mmse,
-    size_t parm_size, MissingAntennaBehavior missing_antenna_behavior) {
+    schaapcommon::h5parm::SolTab* sol_tab2, bool invert, size_t parm_size,
+    MissingAntennaBehavior missing_antenna_behavior) {
   const size_t num_parms = GetNParms(gain_type);
 
   parms_.resize(num_parms, antenna_names.size(), freqs.size() * times.size());
@@ -124,7 +123,7 @@ JonesParameters::JonesParameters(
     }
   }
   if (invert) {
-    Invert(parms_, sigma_mmse, gain_type);
+    Invert(parms_, gain_type);
   }
 }
 
@@ -171,13 +170,13 @@ void JonesParameters::MakeComplex(
         break;
       case GainType::kClock:
         parms_(0, ant, tf) =
-            StablePolar(1., parm_values[0][ant][tf] * freq * casacore::C::_2pi);
+            StablePolar(1., parm_values[0][ant][tf] * freq * 2.0 * M_PI);
         if (parm_values.size() == 1) {  // No Clock:0, only Clock:
-          parms_(1, ant, tf) = StablePolar(
-              1., parm_values[0][ant][tf] * freq * casacore::C::_2pi);
+          parms_(1, ant, tf) =
+              StablePolar(1.0, parm_values[0][ant][tf] * freq * 2.0 * M_PI);
         } else {  // Clock:0 and Clock:1
-          parms_(1, ant, tf) = StablePolar(
-              1., parm_values[1][ant][tf] * freq * casacore::C::_2pi);
+          parms_(1, ant, tf) =
+              StablePolar(1.0, parm_values[1][ant][tf] * freq * 2.0 * M_PI);
         }
         break;
       case GainType::kRotationAngle: {
@@ -314,8 +313,8 @@ void JonesParameters::FillParmValues(
   }
 }
 
-void JonesParameters::Invert(casacore::Cube<casacore::Complex>& parms,
-                             float sigma_mmse, GainType gain_type) {
+void JonesParameters::Invert(casacore::Cube<std::complex<float>>& parms,
+                             GainType gain_type) {
   const size_t tf_size = parms.shape()[2];
   const size_t ant_size = parms.shape()[1];
   for (size_t tf = 0; tf < tf_size; ++tf) {
@@ -326,12 +325,6 @@ void JonesParameters::Invert(casacore::Cube<casacore::Complex>& parms,
       } else if (gain_type == GainType::kFullJones ||
                  gain_type == GainType::kFullJonesRealImaginary) {
         aocommon::MC2x2F v(&parms(0, ant, tf));
-
-        // Add the variance of the nuisance term to the elements on the
-        // diagonal.
-        const float variance = sigma_mmse * sigma_mmse;
-        v[0] += variance;
-        v[3] += variance;
         v.Invert();
         v.AssignTo(&parms(0, ant, tf));
       } else if (gain_type == GainType::kRotationMeasure ||
