@@ -1,11 +1,11 @@
 # SPDX-License-Identifier: LGPL-3.0-only
 
-import radler as rd
-import pytest
-from pytest_lazy_fixtures import lf
-import numpy as np
 import os.path
 
+import numpy as np
+import pytest
+
+import radler as rd
 
 WIDTH = 64
 HEIGHT = 64
@@ -15,7 +15,7 @@ MINOR_ITERATION_COUNT = 1000
 
 
 def radler_perform(radler_object: rd.Radler, minor_iteration_count: int):
-    iteration_number = 0
+    iteration_number = 1
     reached_threshold = radler_object.perform(iteration_number)
     assert reached_threshold == False
     assert radler_object.iteration_number <= minor_iteration_count
@@ -34,16 +34,16 @@ def check_model_image_point_source(
 
 
 @pytest.fixture
-def get_settings():
-    settings = rd.Settings()
-    settings.algorithm_type = rd.AlgorithmType.generic_clean
-    settings.trimmed_image_width = WIDTH
-    settings.trimmed_image_height = HEIGHT
-    settings.pixel_scale.x = PIXEL_SCALE
-    settings.pixel_scale.y = PIXEL_SCALE
-    settings.minor_iteration_count = MINOR_ITERATION_COUNT
-    settings.absolute_threshold = 1e-8
-    return settings
+def settings():
+    radler_settings = rd.Settings()
+    radler_settings.algorithm_type = rd.AlgorithmType.generic_clean
+    radler_settings.trimmed_image_width = WIDTH
+    radler_settings.trimmed_image_height = HEIGHT
+    radler_settings.pixel_scale.x = PIXEL_SCALE
+    radler_settings.pixel_scale.y = PIXEL_SCALE
+    radler_settings.minor_iteration_count = MINOR_ITERATION_COUNT
+    radler_settings.absolute_threshold = 1e-8
+    return radler_settings
 
 
 def get_point_source():
@@ -85,7 +85,6 @@ def get_residual(scale: float, shift_x: int, shift_y: int):
     return residual
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 def test_num_threads(settings):
     """
     Check that only positive, non-zero number of threads are accepted.
@@ -102,7 +101,6 @@ def test_num_threads(settings):
     rd.Radler(settings, psf, residual, model, BEAM_SIZE)
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 def test_input_dtype(settings):
     """
     Check that Radler constructor only accepts numpy arrays of dtype=np.float32
@@ -130,7 +128,6 @@ def test_input_dtype(settings):
     rd.Radler(settings, psf, residual, model, BEAM_SIZE)
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 def test_matching_arrays(settings):
     """
     Check that the Radler constructor only accepts valid numpy arrays that match.
@@ -219,7 +216,6 @@ def test_matching_arrays(settings):
         )
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 def test_require_frequencies(settings):
     """
     Check that Radler requires frequencies when spectral fitting is enabled.
@@ -230,7 +226,6 @@ def test_require_frequencies(settings):
         rd.Radler(settings, image, image, image, BEAM_SIZE)
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 def test_default_args(settings):
     """
     Test calling the Radler constructor without providing optional arguments.
@@ -242,7 +237,6 @@ def test_default_args(settings):
     rd.Radler(settings, psf, residual, model, BEAM_SIZE)
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 @pytest.mark.parametrize(
     "algorithm", [rd.AlgorithmType.generic_clean, rd.AlgorithmType.multiscale]
 )
@@ -289,7 +283,6 @@ def test_point_source(
 
 
 @pytest.mark.parametrize("lm_shift_given", [True, False])
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 def test_write_component_list(settings, lm_shift_given):
     """
     Check writing of component list
@@ -352,7 +345,6 @@ def test_write_component_list(settings, lm_shift_given):
     os.remove(SOURCES_FILENAME)
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 @pytest.mark.parametrize(
     "algorithm", [rd.AlgorithmType.generic_clean, rd.AlgorithmType.multiscale]
 )
@@ -390,7 +382,6 @@ def test_one_entry_worktable(settings, algorithm, scale, source_shift):
     )
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 @pytest.mark.parametrize(
     "algorithm", [rd.AlgorithmType.generic_clean, rd.AlgorithmType.multiscale]
 )
@@ -440,7 +431,6 @@ def test_ndeconvolution_is_noriginal(settings, algorithm):
         )
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 @pytest.mark.parametrize(
     "algorithm", [rd.AlgorithmType.generic_clean, rd.AlgorithmType.multiscale]
 )
@@ -481,7 +471,6 @@ def test_image_cube_non_joined(settings, algorithm):
         )
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 @pytest.mark.parametrize(
     "algorithm", [rd.AlgorithmType.generic_clean, rd.AlgorithmType.multiscale]
 )
@@ -552,7 +541,6 @@ def test_ndeconvolution_lt_noriginal(settings, algorithm):
     np.testing.assert_allclose(models[0], model_image_ref, atol=2e-6)
 
 
-@pytest.mark.parametrize("settings", [lf("get_settings")])
 @pytest.mark.parametrize(
     "algorithm", [rd.AlgorithmType.generic_clean, rd.AlgorithmType.multiscale]
 )
