@@ -627,7 +627,7 @@ template<typename T> class vfmav: public cfmav<T>
       : cfmav<T>(info, d_, buf) {}
 
   public:
-    using tbuf::raw, tbuf::data, tinfo::ndim;
+    using tinfo::ndim;
     vfmav() {}
     vfmav(T *d_, const fmav_info &info)
       : cfmav<T>(d_, info) {}
@@ -835,7 +835,6 @@ template<typename T, size_t ndim> class vmav: public cmav<T, ndim>
   public:
     using typename tinfo::shape_t;
     using typename tinfo::stride_t;
-    using tbuf::raw, tbuf::data;
     using tinfo::contiguous, tinfo::size, tinfo::idx, tinfo::conformable;
 
   protected:
@@ -874,7 +873,7 @@ template<typename T, size_t ndim> class vmav: public cmav<T, ndim>
     using cmav<T, ndim>::to_fmav;
     vfmav<T> to_fmav() const { return operator vfmav<T>(); }
 
-    using parent::operator();
+    //using parent::operator();
     template<typename... Ns> T &operator()(Ns... ns) const
       { return const_cast<T &>(parent::operator()(ns...)); }
 
