@@ -162,6 +162,8 @@ class SolTab : private H5::Group {
 
   std::string GetType() const { return type_; }
 
+  bool Exists(const std::string name) const { return nameExists(name); }
+
   /// Get the values of this SolTab for a given antenna.
   std::vector<double> GetValues(const std::string& ant_name,
                                 size_t starttimeslot, size_t ntime,
@@ -197,10 +199,20 @@ class SolTab : private H5::Group {
                                              size_t pol, size_t dir,
                                              bool nearest) const;
 
-  /// Get the values or weights of this SolTab for a given antenna given an
-  /// antenna name, a direction index, and a (range of) times and frequencies.
-  /// In the returned vector, the freq will be the fastest changing index,
-  /// irrespective of the axis ordering in the underlying h5 data structure.
+  std::tuple<int, int, int, int, int> GetSubArray(
+      const std::string& val_or_weight, const size_t start_antenna_index,
+      const size_t n_antennas, const size_t antenna_step,
+      const size_t start_time_index, const size_t n_times,
+      const size_t time_step, const size_t start_freq_index,
+      const size_t n_freqs, const size_t freq_step,
+      const size_t start_polarization_index, const size_t n_polarizations,
+      const size_t polarization_step, const size_t start_direction_index,
+      const size_t n_directions, const size_t direction_step,
+      std::vector<double>& buffer) const;
+
+  /// @param antenna_name Antenna name. Is ignored if there is no "ant" table.
+  /// @throw std::runtime_error If the "ant" table exists but it does not
+  /// contain @p antenna_name .
   std::vector<double> GetSubArray(const std::string& val_or_weight,
                                   const std::string& antenna_name,
                                   size_t start_time_index, size_t n_times,
@@ -212,6 +224,10 @@ class SolTab : private H5::Group {
       const std::string& value_or_weight, const size_t n_axes,
       const std::array<size_t, 5>& size_axis) const;
 
+  /// Get the values or weights of this SolTab for a given antenna given an
+  /// antenna name, a direction index, and a (range of) times and frequencies.
+  /// In the returned vector, the freq will be the fastest changing index,
+  /// irrespective of the axis ordering in the underlying h5 data structure.
   std::vector<double> GetValuesOrWeights(const std::string& val_or_weight,
                                          const std::string& antenna_name,
                                          const std::vector<double>& times,
