@@ -8,7 +8,7 @@
 
 namespace schaapcommon::reordering {
 
-enum class StorageManagerType { Default, StokesI };
+enum class StorageManagerType { Default, StokesI, Sisco };
 
 inline StorageManagerType GetStorageManagerType(
     const std::string& type_string) {
@@ -17,6 +17,8 @@ inline StorageManagerType GetStorageManagerType(
     return StorageManagerType::Default;
   else if (lowercase_type == "stokes_i" || lowercase_type == "stokes-i")
     return StorageManagerType::StokesI;
+  else if (lowercase_type == "sisco")
+    return StorageManagerType::Sisco;
   else
     throw std::runtime_error("Unknown storage manager type: " + type_string);
 }
