@@ -23,6 +23,7 @@
 #include "../main/settings.h"
 
 #include "averagebeam.h"
+#include "idgcommon.h"
 #include "idgconfiguration.h"
 
 #ifdef HAVE_EVERYBEAM
@@ -55,7 +56,7 @@ IdgMsGridder::IdgMsGridder(const Settings& settings, const Resources& resources,
       _buffersize(0),
       _resources(resources) {
   IdgConfiguration::Read(_proxyType, _buffersize, _options);
-  setIdgType();
+  _proxyType = GetIdgType(GetSettings());
   _bufferset = std::unique_ptr<idg::api::BufferSet>(
       idg::api::BufferSet::create(_proxyType));
   if (settings.gridWithBeam || !settings.atermConfigFilename.empty())
@@ -65,11 +66,6 @@ IdgMsGridder::IdgMsGridder(const Settings& settings, const Resources& resources,
     _options["taper"] = std::string("blackman-harris");
 }
 
-IdgMsGridder::~IdgMsGridder() {
-  Logger::Info << "Gridding: " << _griddingWatch.ToString()
-               << ", degridding: " << _degriddingWatch.ToString() << '\n';
-}
-
 size_t IdgMsGridder::GetNInversionPasses() const {
   if (GetPsfMode() == PsfMode::kNone && _averageBeam->Empty()) {
     // Two passes are used for this case: i) average beam creation; ii)
@@ -178,14 +174,21 @@ size_t IdgMsGridder::GridMeasurementSet(
     return 0;
 #endif
 
+  // TODO For now we do not allow multiple bands in one msprovider
+  if (!ms_data.ms_provider->IsRegular()) {
+    throw std::runtime_error(
+        "IDG implementation does not support irregular data yet");
+  }
+  // Regular data should always have one band...
+  assert(_selectedBands.BandCount() == 1);
+  const aocommon::BandData& selected_band = *_selectedBands.begin();
+
   const size_t n_vis_polarizations = ms_data.ms_provider->NPolarizations();
   constexpr size_t n_idg_polarizations = 4;
-  const size_t data_size = _selectedBand.ChannelCount() * n_idg_polarizations;
+  const size_t data_size = selected_band.ChannelCount() * n_idg_polarizations;
   aocommon::UVector<float> weight_buffer(data_size);
   aocommon::UVector<std::complex<float>> model_buffer(data_size);
-  aocommon::UVector<bool> selection_buffer(_selectedBand.ChannelCount(), true);
-
-  _griddingWatch.Start();
+  aocommon::UVector<bool> selection_buffer(selected_band.ChannelCount(), true);
 
   // The gridder doesn't need to know the absolute time index; this value
   // indexes relatively to where we start in the measurement set, and only
@@ -213,8 +216,8 @@ size_t IdgMsGridder::GridMeasurementSet(
       if (aterm_maker) {
         timestep_reader.GetUVWsForTimestep(uvws);
         if (aterm_maker->Calculate(aterm_buffer.data(), current_time,
-                                   _selectedBand.CentreFrequency(),
-                                   metadata.fieldId, uvws.data())) {
+                                   selected_band.CentreFrequency(),
+                                   metadata.field_id, uvws.data())) {
           _bufferset->get_gridder(kGridderIndex)
               ->set_aterm(time_index, aterm_buffer.data());
           Logger::Debug << "Calculated a-terms for timestep " << time_index
@@ -224,19 +227,27 @@ size_t IdgMsGridder::GridMeasurementSet(
 #endif
     }
 
-    row_data.uvw[0] = metadata.uInM;
-    row_data.uvw[1] = metadata.vInM;
-    row_data.uvw[2] = metadata.wInM;
+    row_data.uvw[0] = metadata.u_in_m;
+    row_data.uvw[1] = metadata.v_in_m;
+    row_data.uvw[2] = metadata.w_in_m;
 
     row_data.antenna1 = metadata.antenna1;
     row_data.antenna2 = metadata.antenna2;
-    row_data.timeIndex = time_index;
+    row_data.time_index = time_index;
 
+    const size_t n_parms = NumValuesPerSolution();
     if (n_vis_polarizations == 1) {
-      GetInstrumentalVisibilities<1>(*ms_reader, ms_data.antenna_names.size(),
-                                     row_data, _selectedBand,
-                                     weight_buffer.data(), model_buffer.data(),
-                                     selection_buffer.data(), metadata);
+      if (n_parms == 2) {
+        GetInstrumentalVisibilities<1, 2>(
+            *ms_reader, ms_data.antenna_names.size(), row_data,
+            weight_buffer.data(), model_buffer.data(), selection_buffer.data(),
+            metadata);
+      } else {
+        GetInstrumentalVisibilities<1, 4>(
+            *ms_reader, ms_data.antenna_names.size(), row_data,
+            weight_buffer.data(), model_buffer.data(), selection_buffer.data(),
+            metadata);
+      }
       // The data is placed in the first quarter of the buffers: reverse copy it
       // and expand it to 4 polarizations. TODO at a later time, IDG should
       // be able to directly accept 1 polarization instead of 4.
@@ -253,10 +264,17 @@ size_t IdgMsGridder::GridMeasurementSet(
         source_index--;
       }
     } else if (n_vis_polarizations == 2) {
-      GetInstrumentalVisibilities<2>(*ms_reader, ms_data.antenna_names.size(),
-                                     row_data, _selectedBand,
-                                     weight_buffer.data(), model_buffer.data(),
-                                     selection_buffer.data(), metadata);
+      if (n_parms == 2) {
+        GetInstrumentalVisibilities<2, 2>(
+            *ms_reader, ms_data.antenna_names.size(), row_data,
+            weight_buffer.data(), model_buffer.data(), selection_buffer.data(),
+            metadata);
+      } else {
+        GetInstrumentalVisibilities<2, 4>(
+            *ms_reader, ms_data.antenna_names.size(), row_data,
+            weight_buffer.data(), model_buffer.data(), selection_buffer.data(),
+            metadata);
+      }
       // The data is placed in the first half of the buffers: reverse copy it
       // and expand it to 4 polarizations. TODO at a later time, IDG should
       // be able to directly accept 2 pols instead of 4.
@@ -274,14 +292,21 @@ size_t IdgMsGridder::GridMeasurementSet(
       }
     } else {
       assert(n_vis_polarizations == 4);
-      GetInstrumentalVisibilities<4>(*ms_reader, ms_data.antenna_names.size(),
-                                     row_data, _selectedBand,
-                                     weight_buffer.data(), model_buffer.data(),
-                                     selection_buffer.data(), metadata);
+      if (n_parms == 2) {
+        GetInstrumentalVisibilities<4, 2>(
+            *ms_reader, ms_data.antenna_names.size(), row_data,
+            weight_buffer.data(), model_buffer.data(), selection_buffer.data(),
+            metadata);
+      } else {
+        GetInstrumentalVisibilities<4, 4>(
+            *ms_reader, ms_data.antenna_names.size(), row_data,
+            weight_buffer.data(), model_buffer.data(), selection_buffer.data(),
+            metadata);
+      }
     }
 
-    row_data.uvw[1] = -metadata.vInM;  // DEBUG vdtol, flip axis
-    row_data.uvw[2] = -metadata.wInM;  //
+    row_data.uvw[1] = -metadata.v_in_m;  // DEBUG vdtol, flip axis
+    row_data.uvw[2] = -metadata.w_in_m;  //
 
     _bufferset->get_gridder(kGridderIndex)
         ->grid_visibilities(time_index, metadata.antenna1, metadata.antenna2,
@@ -291,8 +316,6 @@ size_t IdgMsGridder::GridMeasurementSet(
   }
   _bufferset->finished();
 
-  _griddingWatch.Pause();
-
   return 0;
 }
 
@@ -357,22 +380,6 @@ void IdgMsGridder::StartPredict(std::vector<Image>&& images) {
 
 void IdgMsGridder::FinishPredict() {}
 
-void IdgMsGridder::setIdgType() {
-  switch (GetSettings().idgMode) {
-    default:
-      return;
-    case Settings::IDG_CPU:
-      _proxyType = idg::api::Type::CPU_OPTIMIZED;
-      return;
-    case Settings::IDG_GPU:
-      _proxyType = idg::api::Type::CUDA_GENERIC;
-      return;
-    case Settings::IDG_HYBRID:
-      _proxyType = idg::api::Type::HYBRID_CUDA_CPU_OPTIMIZED;
-      return;
-  }
-}
-
 size_t IdgMsGridder::PredictMeasurementSet(
     const MsProviderCollection::MsData& ms_data) {
   aocommon::UVector<std::complex<float>> aterm_buffer;
@@ -391,10 +398,17 @@ size_t IdgMsGridder::PredictMeasurementSet(
 
   _outputProvider = ms_data.ms_provider;
 
+  // TODO For now we do not allow multiple bands in one msprovider
+  if (!ms_data.ms_provider->IsRegular())
+    throw std::runtime_error(
+        "IDG implementation does not support irregular data yet");
+  // Regular data should always have one band...
+  assert(_selectedBands.BandCount() == 1);
+  const aocommon::BandData& selected_band = *_selectedBands.begin();
+
   constexpr size_t n_idg_polarizations = 4;
-  aocommon::UVector<std::complex<float>> buffer(_selectedBand.ChannelCount() *
+  aocommon::UVector<std::complex<float>> buffer(selected_band.ChannelCount() *
                                                 n_idg_polarizations);
-  _degriddingWatch.Start();
 
   int time_index = -1;
   double current_time = -1.0;
@@ -419,8 +433,8 @@ size_t IdgMsGridder::PredictMeasurementSet(
       if (aterm_maker) {
         timestep_reader.GetUVWsForTimestep(uvws);
         if (aterm_maker->Calculate(aterm_buffer.data(), current_time,
-                                   _selectedBand.CentreFrequency(),
-                                   metadata.fieldId, uvws.data())) {
+                                   selected_band.CentreFrequency(),
+                                   metadata.field_id, uvws.data())) {
           _bufferset->get_degridder(kGridderIndex)
               ->set_aterm(time_index, aterm_buffer.data());
           Logger::Debug << "Calculated new a-terms for timestep " << time_index
@@ -431,13 +445,13 @@ size_t IdgMsGridder::PredictMeasurementSet(
     }
 
     IDGPredictionRow row;
-    row.uvw[0] = metadata.uInM;
-    row.uvw[1] = -metadata.vInM;
-    row.uvw[2] = -metadata.wInM;
+    row.uvw[0] = metadata.u_in_m;
+    row.uvw[1] = -metadata.v_in_m;
+    row.uvw[2] = -metadata.w_in_m;
     row.antenna1 = metadata.antenna1;
     row.antenna2 = metadata.antenna2;
-    row.timeIndex = time_index;
-    row.rowId = provRowId;
+    row.time_index = time_index;
+    row.row_id = provRowId;
     predictRow(row, ms_data.antenna_names);
   }
 
@@ -448,7 +462,7 @@ size_t IdgMsGridder::PredictMeasurementSet(
 void IdgMsGridder::predictRow(IDGPredictionRow& row,
                               const std::vector<std::string>& antenna_names) {
   while (_bufferset->get_degridder(kGridderIndex)
-             ->request_visibilities(row.rowId, row.timeIndex, row.antenna1,
+             ->request_visibilities(row.row_id, row.time_index, row.antenna1,
                                     row.antenna2, row.uvw)) {
     computePredictionBuffer(antenna_names);
   }
@@ -461,29 +475,32 @@ void IdgMsGridder::computePredictionBuffer(
   const size_t n_vis_polarizations = _outputProvider->NPolarizations();
   for (std::pair<long unsigned, std::complex<float>*>& row :
        available_row_ids) {
-    MSProvider::MetaData metaData;
-    ReadPredictMetaData(metaData);
+    MSProvider::MetaData metadata;
+    ReadPredictMetaData(metadata);
+    const aocommon::BandData& band = _selectedBands[metadata.data_desc_id];
     if (n_vis_polarizations == 1) {
       // Place Stokes I in the first quarter of the array
-      for (size_t i = 0; i != _selectedBand.ChannelCount(); ++i) {
+      for (size_t i = 0; i != band.ChannelCount(); ++i) {
         row.second[i] = (row.second[i * 4] + row.second[i * 4 + 3]) / 2.0f;
       }
 
     } else if (n_vis_polarizations == 2) {
       // Remove the XY/YX pols from the data and place the result in the first
       // half of the array
-      for (size_t i = 0; i != _selectedBand.ChannelCount(); ++i) {
+      for (size_t i = 0; i != band.ChannelCount(); ++i) {
         row.second[i * 2] = row.second[i * 4];
         row.second[i * 2 + 1] = row.second[i * 4 + 3];
       }
     } else {
       assert(n_vis_polarizations == 4);
     }
+    const double* uvw = nullptr;
     WriteInstrumentalVisibilities(*_outputProvider, antenna_names.size(),
-                                  _selectedBand, row.second, metaData);
+                                  metadata.data_desc_id, row.second, uvw,
+                                  metadata.field_id, metadata.antenna1,
+                                  metadata.antenna2, metadata.time);
   }
   _bufferset->get_degridder(kGridderIndex)->finished_reading();
-  _degriddingWatch.Pause();
 }
 
 std::vector<Image> IdgMsGridder::ResultImages() {
@@ -580,19 +597,27 @@ bool IdgMsGridder::prepareForMeasurementSet(
     aocommon::UVector<std::complex<float>>& aTermBuffer,
     idg::api::BufferSetType bufferSetType) {
 #endif  // HAVE_EVERYBEAM
-  const float max_baseline = ms_data.max_baseline_meters;
+  const float max_baseline = ms_data.max_baseline_in_m;
   // Skip this ms if there is no data in it
   if (!max_baseline) return false;
 
-  _selectedBand = ms_data.SelectedBand();
+  _selectedBands = ms_data.ms_provider->SelectedBands();
+
+  // TODO For now we do not allow multiple bands in one msprovider
+  if (!ms_data.ms_provider->IsRegular())
+    throw std::runtime_error(
+        "IDG implementation does not support irregular data yet");
+  // Regular data should always have one band...
+  assert(_selectedBands.BandCount() == 1);
+  const aocommon::BandData& selected_band = *_selectedBands.begin();
 
   // TODO for now we map the ms antennas directly to the gridder's antenna,
   // including non-selected antennas. Later this can be made more efficient.
-  const size_t nStations = ms_data.ms_provider->MS()->antenna().nrow();
+  const size_t nStations = ms_data.ms_provider->NAntennas();
 
   std::vector<std::vector<double>> bands;
-  bands.emplace_back(_selectedBand.begin(), _selectedBand.end());
-  const size_t nChannels = _selectedBand.ChannelCount();
+  bands.emplace_back(selected_band.begin(), selected_band.end());
+  const size_t nChannels = selected_band.ChannelCount();
 
   // Only one-third of the mem is allocated to the buffers, so that memory
   // remains available for the images and other things done by IDG.
@@ -649,8 +674,7 @@ bool IdgMsGridder::prepareForMeasurementSet(
 #ifdef HAVE_EVERYBEAM
 std::unique_ptr<class ATermBase> IdgMsGridder::getATermMaker(
     const MsProviderCollection::MsData& ms_data) {
-  SynchronizedMS ms = ms_data.ms_provider->MS();
-  size_t nr_stations = ms->antenna().nrow();
+  const size_t nr_stations = ms_data.ms_provider->NAntennas();
   if (!GetSettings().atermConfigFilename.empty() ||
       GetSettings().gridWithBeam) {
     // IDG uses a flipped coordinate system which is moved by half a pixel:
@@ -673,10 +697,17 @@ std::unique_ptr<class ATermBase> IdgMsGridder::getATermMaker(
     aterm_settings.trimmed_image_width = GetSettings().trimmedImageWidth;
     aterm_settings.max_support = GetSettings().atermKernelSize;
 
+    std::optional<SynchronizedMS> optional_ms =
+        ms_data.ms_provider->MsIfAvailable();
+    if (!optional_ms) {
+      throw std::runtime_error(
+          "Gridding with beam or a-term configuration file is currently not "
+          "supported without an on-disk measurement set");
+    }
     if (!GetSettings().atermConfigFilename.empty()) {
       ParsetReader parset_aterms(GetSettings().atermConfigFilename);
       // If MWA MS and "beam" in aterms, get the path to the coefficient file
-      if (everybeam::GetTelescopeType(*ms) ==
+      if (everybeam::GetTelescopeType(**optional_ms) ==
           everybeam::TelescopeType::kMWATelescope) {
         std::vector<std::string> aterms = parset_aterms.GetStringList("aterms");
         if (std::find(aterms.begin(), aterms.end(), "beam") != aterms.end()) {
@@ -688,11 +719,11 @@ std::unique_ptr<class ATermBase> IdgMsGridder::getATermMaker(
       std::unique_ptr<ATermConfig> config(
           new ATermConfig(nr_stations, system, aterm_settings));
       config->SetSaveATerms(GetSettings().saveATerms, GetSettings().prefixName);
-      config->Read(*ms, parset_aterms, ms.Filename());
+      config->Read(**optional_ms, parset_aterms, optional_ms->Filename());
       return config;
     } else {
       // If MWA MS, get the path to the coefficient file
-      if (everybeam::GetTelescopeType(*ms) ==
+      if (everybeam::GetTelescopeType(**optional_ms) ==
           everybeam::TelescopeType::kMWATelescope) {
         aterm_settings.coeff_path =
             wsclean::mwa::FindCoeffFile(GetSettings().mwaPath);
@@ -704,7 +735,7 @@ std::unique_ptr<class ATermBase> IdgMsGridder::getATermMaker(
                                              : "DEFAULT";
 
       std::unique_ptr<ATermBeam> beam = ATermConfig::GetATermBeam(
-          *ms, system, aterm_settings, frequencyInterpolation,
+          **optional_ms, system, aterm_settings, frequencyInterpolation,
           GetSettings().beamNormalisationMode, useChannelFrequency,
           elementResponseModel, GetSettings().beamMode);
       beam->SetSaveATerms(GetSettings().saveATerms, GetSettings().prefixName);
