@@ -21,6 +21,7 @@
 #include <iostream>
 #include <optional>
 #include <string>
+#include <string_view>
 #include <sstream>
 
 using aocommon::Logger;
@@ -275,6 +276,10 @@ Options can be:
    Always make the psf, even when no cleaning is performed.
 -make-psf-only
    Only make the psf, no images are made.
+-skip-final-iteration
+   Skip the prediction-gridding round after finishing deconvolution. The final images may have slightly lower
+   quality, but the model (images/list) is complete. This is therefore useful in self-calibration when the
+   model image or model component list is used instead of the visibilities.
 -visibility-weighting-mode [normal/squared/unit]
    Specify visibility weighting modi. Affects how the weights (normally) stored in
    WEIGHT_SPECTRUM column are applied. Useful for estimating e.g. EoR power spectra errors.
@@ -308,10 +313,23 @@ Options can be:
 -wgridder-accuracy <value>
    Set the w-gridding accuracy. Default: 1e-4
    Useful range: 1e-2 to 1e-6
+-wtowers-subgrid-size <value>
+   Set w-towers subgrid size. Default: 256
+-wtowers-kernel-size <value>
+-wtowers-w-kernel-size <value>
+-wtowers-padding <value>
+-wtowers-w-padding <value>
+   Gridding antialiasing kernel sizes and padding. Defaults: 8, 8, 1.5, 1.5.
+-wtowers-accuracy <value>
+   Set the w-towers accuracy for the purpose of w-tower height calculation. Default: 0
+   When zero, w-towers will set it to the gridder's accuracy (depends on kernel size)
+   Useful range: 1e-2 to 1e-4 (will fail if base accuracy of gridder is worse!)
 -compound-tasks
    Schedule compound gridding tasks which contain all facets for a single image.
 -shared-facet-reads
    When parallel gridding with multiple facets read data only once per compound gridding task into a shared data buffer and share this buffer for the gridders of all facets within the task. Implicitly sets -compound-tasks.
+-shared-facet-writes
+   When parallel gridding with multiple facets, sum all predicted data from all facets in a compound gridding task in a single shared memory data buffer and then write this buffer once. This can be more efficient than having each facet write independently. Implicitly sets -compound-tasks.
 
   ** A-TERM GRIDDING **
 -aterm-config <filename>
@@ -329,6 +347,11 @@ Options can be:
    Apply solutions from the provided (h5) file per facet when gridding facet based images.
    Provided file is assumed to be in H5Parm format.
    Filename is followed by a comma separated list of strings specifying which sol tabs from the provided H5Parm file are used.
+-apply-time-frequency-smearing
+   Takes time frequency smearing over the integration interval of the visibilities into account
+   in the computation of the psf and when predicting visibilities. This only affects directions
+   away from the phase centre, so this option only has an effect when
+   the -dd-psf and/or the -facet-regions option is enabled.
 -no-solution-directions-check
    Disable the check that requires the number of solution directions to be equal to the number of directions in the region
    file.
@@ -378,7 +401,7 @@ Options can be:
    Column to which the predicted data is written. Default: MODEL_DATA.
 -model-storage-manager <type>
    If a new model column needs to be written, create the model using this storage manager. Supported types:
-   "default" and "stokes-i".
+   "default", "stokes-i" and "sisco".
 -maxuvw-m <meters>
 -minuvw-m <meters>
    Set the min/max baseline distance in meters.
@@ -404,6 +427,8 @@ Options can be:
    the threshold is reached, continue cleaning with the mask down to the normal threshold.
 -abs-auto-mask <abs-threshold>
    Absolute stopping threshold for the mask generation stage. See -auto-mask.
+-auto-mask-nmiter <max iterations>
+   Limit for the number of major iterations after having reached the auto-mask threshold. Default: 2.
 -local-rms
    Instead of using a single RMS for auto thresholding/masking, use a spatially varying
    RMS image.
@@ -419,6 +444,16 @@ Options can be:
 -mgain <gain>
    Cleaning gain for major iterations: Ratio of peak that will be subtracted in each major
    iteration. To use major iterations, 0.85 is a good value. Default: 1.0
+-mgain-boosting <boost value>
+   Values higher than 1 will cause a higher mgain value in the first two iterations. The first
+   iterations uses mgain' = 1 - (1 - mgain) ^ boost_value, the second iteration uses half the boost.
+   Default: 1.0
+-major-iteration-mode <single/dual/full>
+   Alters the behaviour of major iterations when using auto-masking. In single mode, every iteration
+   ends once the mgain is reached. In dual mode, during the mask-creation stage, a second major iteration
+   is started in which the so-far constructed mask is used to constrain the cleaning. Full mode is
+   like dual, but ignores mgain in the repeated iteration. Dual was found to be a good average strategy,
+   but can occasionally be too aggressive. If cleaning diverges, single is recommended. Default: dual.
 -join-polarizations
    Perform deconvolution by searching for peaks in the sum of squares of the polarizations,
    but subtract components from the individual images. Only possible when imaging two or four Stokes
@@ -430,6 +465,9 @@ Options can be:
    Perform deconvolution by searching for peaks in the MF image,
 but subtract components from individual channels.
    This will turn on mf-weighting by default. Default: off.
+-component-optimization <mode>
+   Optimization algorithm for determining component levels after the auto mask threshold has been reached.
+   Options: clean, gradient-descent, linear-solver.
 -spectral-correction <reffreq> <term list>
    Enable correction of the given spectral function inside deconvolution.
    This can e.g. avoid downweighting higher frequencies because of
@@ -459,7 +497,7 @@ but subtract components from individual channels.
 -multiscale-convolution-padding <padding>
    Size of zero-padding for convolutions during the multi-scale cleaning. Default: 1.1
 -asp
-   Use the adaptive scale pixel algorithm.   
+   Use the adaptive scale pixel algorithm.
 -no-multiscale-fast-subminor
    Disable the 'fast subminor loop' optimization, that will only search a part of the
    image during the multi-scale subminor loop. The optimization is on by default.
@@ -503,6 +541,9 @@ but subtract components from individual channels.
    when the channels are joined with -join-channels.
 -fit-spectral-log-pol <nterms>
    Like fit-spectral-pol, but fits a logarithmic polynomial over frequency instead.
+-fit-rm
+   Fits each spectrum to a wavelength-squared sinusodial signal, thereby performing RM synthesis
+   during cleaning. This option only works when imaging the Stokes Q and U polarizations.
 -force-spectrum <fitsfile>
    Uses the fits file to force spectral indices (or other/more terms)   during the deconvolution.
 -deconvolution-channels <nchannels>
@@ -518,8 +559,17 @@ but subtract components from individual channels.
 -deconvolution-threads <n>
    Number of threads to use during deconvolution. On machines with a large nr of cores, this may be used to decrease the memory usage.
    If not specified, the number of threads during deconvolution is controlled with the -j option.
-
-  ** RESTORATION OPTIONS **
+  ** SKY MODEL DRAWING OPTIONS**
+-draw-model <input model>
+   Create a FITS image from a sky model, provided in the BBS/DP3 text format. Use -name to specify the prefix of the output image(s).
+-draw-centre <ra> <dec>
+  Draw the sky model at the given phase centre. Takes precedent over observation phase centre, if a MS is provided.
+-draw-frequencies <central frequency> <bandwidth>
+   Sets the central fequency and bandwidth of the image that is to be rendered (in Hz).
+-sinc-window-size <window size in pixels>
+   Sinc convolution window size (in pixels). Default: 127.
+-draw-spectral-terms <nterms>
+   Number of spectral terms to draw; for each term a seperate image will be rendered. Defaults to 1, to only draw the first term image.
 -restore <input residual> <input model> <output image>
    Restore the model image onto the residual image and save it in output image. By
    default, the beam parameters are read from the residual image. If this parameter
@@ -539,6 +589,8 @@ but subtract components from individual channels.
    Do not determine beam shape from the PSF.
 -beam-fitting-size <factor>
    Use a fitting box the size of <factor> times the theoretical beam size for fitting a Gaussian to the PSF.
+-fit-beam-with-negatives / -fit-beam-without-negatives
+   Include or don't include negative values when fitting a Gaussian to the PSF. Default: with negatives.
 -theoretic-beam
    Write the beam in output fits files as calculated from the longest projected baseline.
    This method results in slightly less accurate beam size/integrated fluxes, but provides a beam size
@@ -647,6 +699,9 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
         Logger::Info << "IDG is available.\n";
 #endif
         Logger::Info << "WGridder is available.\n";
+#ifdef BUILD_WTOWERS
+        Logger::Info << "W-Towers is available.\n";
+#endif
       }
       return false;
     } else if (param == "help") {
@@ -758,6 +813,23 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "mgain") {
       IncArgi(argi, argc);
       settings.deconvolutionMGain = ParseDouble(argv[argi], 0.0, "mgain");
+    } else if (param == "mgain-boosting") {
+      IncArgi(argi, argc);
+      settings.deconvolutionBoosting =
+          ParseDouble(argv[argi], 0.0, "mgain-boosting", false);
+    } else if (param == "major-iteration-mode") {
+      IncArgi(argi, argc);
+      const std::string_view mode(argv[argi]);
+      if (mode == "single")
+        settings.majorIterationStrategy =
+            radler::MajorIterationStrategy::kNormal;
+      else if (mode == "dual")
+        settings.majorIterationStrategy = radler::MajorIterationStrategy::kDual;
+      else if (mode == "full")
+        settings.majorIterationStrategy = radler::MajorIterationStrategy::kFull;
+      else
+        throw std::runtime_error(
+            "Invalid major iteration strategy specified: " + std::string(mode));
     } else if (param == "niter") {
       IncArgi(argi, argc);
       settings.deconvolutionIterationCount = ParseSizeT(argv[argi], "niter");
@@ -780,6 +852,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       IncArgi(argi, argc);
       settings.absoluteAutoMaskThreshold = FluxDensity::Parse(
           argv[argi], "absolute threshold parameter", FluxDensity::kJansky);
+    } else if (param == "auto-mask-nmiter") {
+      IncArgi(argi, argc);
+      settings.majorAutoMaskIterations =
+          ParseSizeT(argv[argi], "auto-mask-nmiter");
     } else if (param == "local-rms") {
       settings.localRMSMethod = radler::LocalRmsMethod::kRmsWindow;
     } else if (param == "local-rms-strength") {
@@ -920,6 +996,8 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.makePSF = true;
     } else if (param == "make-psf-only") {
       settings.makePSFOnly = true;
+    } else if (param == "skip-final-iteration") {
+      settings.skipFinalIteration = true;
     } else if (param == "name") {
       IncArgi(argi, argc);
       settings.prefixName = argv[argi];
@@ -977,6 +1055,21 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
           aocommon::Polarization::ParseList(argv[argi]);
     } else if (param == "join-channels") {
       settings.joinedFrequencyDeconvolution = true;
+    } else if (param == "component-optimization") {
+      IncArgi(argi, argc);
+      const std::string mode = boost::to_lower_copy(std::string(argv[argi]));
+      if (mode == "clean")
+        settings.componentOptimizationAlgorithm =
+            radler::OptimizationAlgorithm::kClean;
+      else if (mode == "linear-solver")
+        settings.componentOptimizationAlgorithm =
+            radler::OptimizationAlgorithm::kLinearEquationSolver;
+      else if (mode == "gradient-descent")
+        settings.componentOptimizationAlgorithm =
+            radler::OptimizationAlgorithm::kGradientDescent;
+      else
+        throw std::runtime_error("Invalid value for component-optimization: " +
+                                 mode);
     } else if (param == "mf-weighting" || param == "mfs-weighting") {
       mfWeighting = true;
       // mfs was renamed to mf in wsclean 2.7
@@ -1092,6 +1185,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
           schaapcommon::fitters::SpectralFittingMode::kLogPolynomial;
       settings.spectralFittingTerms =
           ParseSizeT(argv[argi], "fit-spectral-log-pol");
+    } else if (param == "fit-rm") {
+      settings.spectralFittingMode =
+          schaapcommon::fitters::SpectralFittingMode::kRotationMeasure;
+      settings.spectralFittingTerms = 1;
     } else if (param == "force-spectrum") {
       IncArgi(argi, argc);
       settings.forcedSpectrumFilename = argv[argi];
@@ -1138,6 +1235,29 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       IncArgi(argi, argc);
       settings.weightMode.SetSuperWeight(
           ParseDouble(argv[argi], 0.0, "super-weight"));
+    } else if (param == "draw-model") {
+      settings.mode = Settings::DrawModelMode;
+      IncArgi(argi, argc);
+      settings.inputSkyModelFilename = argv[argi];
+    } else if (param == "draw-centre") {
+      settings.useManualPhaseCentre = true;
+      IncArgi(argi, argc);
+      settings.drawnPhaseCentreRA = aocommon::RaDecCoord::ParseRA(argv[argi]);
+      IncArgi(argi, argc);
+      settings.drawnPhaseCentreDec = aocommon::RaDecCoord::ParseDec(argv[argi]);
+    } else if (param == "draw-frequencies") {
+      IncArgi(argi, argc);
+      settings.drawnSkyModelFrequency =
+          ParseDouble(argv[argi], "centre frequency");
+      IncArgi(argi, argc);
+      settings.drawnSkyModelBandwidth = ParseDouble(argv[argi], "bandwidth");
+    } else if (param == "sinc-window-size") {
+      IncArgi(argi, argc);
+      settings.sincWindowSize = ParseSizeT(argv[argi], "sinc window size");
+    } else if (param == "draw-spectral-terms") {
+      IncArgi(argi, argc);
+      settings.drawnSpectralTermCount =
+          ParseSizeT(argv[argi], "spectral terms");
     } else if (param == "restore" || param == "restore-list") {
       if (param == "restore")
         settings.mode = Settings::RestoreMode;
@@ -1180,6 +1300,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.circularBeam = true;
     } else if (param == "elliptical-beam") {
       settings.circularBeam = false;
+    } else if (param == "fit-beam-with-negatives") {
+      settings.fitBeamWithNegatives = true;
+    } else if (param == "fit-beam-without-negatives") {
+      settings.fitBeamWithNegatives = false;
     } else if (param == "reuse-reordered") {
       settings.reuseReorder = true;
       settings.forceReorder = true;
@@ -1259,6 +1383,9 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "shared-facet-reads") {
       settings.compound_tasks = true;
       settings.shared_facet_reads = true;
+    } else if (param == "shared-facet-writes") {
+      settings.compound_tasks = true;
+      settings.shared_facet_writes = true;
     } else if (param == "aterm-config") {
       IncArgi(argi, argc);
       settings.atermConfigFilename = argv[argi];
@@ -1293,6 +1420,8 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.visibilityReadMode = VisibilityReadMode::kDiagonal;
     } else if (param == "apply-facet-beam") {
       settings.applyFacetBeam = true;
+    } else if (param == "apply-time-frequency-smearing") {
+      settings.applyTimeFrequencySmearing = true;
     } else if (param == "facet-beam-update") {
       IncArgi(argi, argc);
       settings.facetBeamUpdateTime =
@@ -1343,8 +1472,18 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
 #endif
         settings.gridderType = GridderType::IDG;
         settings.minGridResolution = false;
+      } else if (gridder_str == "facet-idg") {
+#if !defined(HAVE_IDG)
+        throw std::runtime_error(
+            "WSClean was not compiled with IDG: to use it, install IDG and "
+            "recompile WSClean");
+#endif
+        settings.gridderType = GridderType::FacetIDG;
+        settings.minGridResolution = false;
       } else if (gridder_str == "wgridder") {
         settings.gridderType = GridderType::WGridder;
+      } else if (gridder_str == "wtowers") {
+        settings.gridderType = GridderType::WTowers;
       } else if (gridder_str == "tuned-wgridder") {
         settings.gridderType = GridderType::TunedWGridder;
       } else if (gridder_str == "wstacking") {
@@ -1380,8 +1519,32 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.gridderType = GridderType::WGridder;
     } else if (param == "wgridder-accuracy") {
       IncArgi(argi, argc);
-      settings.wgridderAccuracy =
+      settings.gridder_accuracy =
           ParseDouble(argv[argi], 0.0, "wgridder-accuracy", false);
+    } else if (param == "wtowers-accuracy") {
+      IncArgi(argi, argc);
+      settings.gridder_accuracy =
+          ParseDouble(argv[argi], 0.0, "wtowers-accuracy", false);
+    } else if (param == "wtowers-subgrid-size") {
+      IncArgi(argi, argc);
+      settings.wtowers_subgrid_size = int(
+          ceil(ParseDouble(argv[argi], 0.0, "wtowers-subgrid-size", false)));
+    } else if (param == "wtowers-support") {
+      IncArgi(argi, argc);
+      settings.wtowers_support =
+          int(ceil(ParseDouble(argv[argi], 0.0, "wtowers-support", false)));
+    } else if (param == "wtowers-w-support") {
+      IncArgi(argi, argc);
+      settings.wtowers_w_support =
+          int(ceil(ParseDouble(argv[argi], 0.0, "wtowers-w-support", false)));
+    } else if (param == "wtowers-padding") {
+      IncArgi(argi, argc);
+      settings.wtowers_padding =
+          ParseDouble(argv[argi], 0.0, "wtowers-padding", false);
+    } else if (param == "wtowers-w-padding") {
+      IncArgi(argi, argc);
+      settings.wtowers_padding =
+          ParseDouble(argv[argi], 0.0, "wtowers-w-padding", false);
     } else if (param == "no-dirty") {
       settings.isDirtySaved = false;
     } else if (param == "save-first-residual") {
@@ -1394,7 +1557,8 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
   }
 
   if (argi == argc && settings.mode != Settings::RestoreMode &&
-      settings.mode != Settings::RestoreListMode)
+      settings.mode != Settings::RestoreListMode &&
+      settings.mode != Settings::DrawModelMode)
     throw std::runtime_error("No input measurement sets given.");
 
   // Done parsing.
@@ -1409,6 +1573,17 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
         "-shared-facet-reads are currently only compatible with -gridder "
         "wgridder");
   }
+  if (settings.shared_facet_writes &&
+      settings.gridderType != GridderType::WGridder) {
+    throw std::runtime_error(
+        "-shared_facet_writes are currently only compatible with -gridder "
+        "wgridder");
+  }
+
+  if (settings.gridder_accuracy == 0.0f &&
+      settings.gridderType == GridderType::WGridder) {
+    settings.gridder_accuracy = 1e-4;
+  }
 
   const size_t defaultAtermSize = settings.atermConfigFilename.empty() ? 5 : 16;
   settings.atermKernelSize = atermKernelSize.value_or(defaultAtermSize);
@@ -1459,6 +1634,9 @@ void CommandLine::Run(class WSClean& wsclean) {
     case Settings::ImagingMode:
       wsclean.RunClean();
       break;
+    case Settings::DrawModelMode:
+      wsclean.DrawModel();
+      break;
   }
 }
 
