@@ -13,8 +13,7 @@ ProgressBar::ProgressBar(const std::string& taskDescription)
 
 ProgressBar::~ProgressBar() { SetProgress(1, 1); }
 
-ProgressBar& ProgressBar::operator=(ProgressBar&& rhs) {
-  SetProgress(1, 1);
+ProgressBar& ProgressBar::operator=(ProgressBar&& rhs) noexcept {
   _displayedDots = rhs._displayedDots;
   _taskDescription = std::move(rhs._taskDescription);
   rhs._displayedDots = 50;
