@@ -12,6 +12,8 @@
 #include "../io/facetreader.h"
 
 using aocommon::Logger;
+using aocommon::Polarization;
+using aocommon::PolarizationEnum;
 
 namespace wsclean {
 
@@ -81,16 +83,17 @@ void Settings::Validate() const {
     }
     if (visibilityReadMode == VisibilityReadMode::kDiagonal &&
         (polarizations.size() != 1 ||
-         *polarizations.begin() != aocommon::PolarizationEnum::StokesI)) {
+         *polarizations.begin() != PolarizationEnum::StokesI)) {
       throw std::runtime_error(
           "-diagonal-visibilities can only be used when making Stokes I "
           "images");
     }
   }
 
-  if (facetRegionFilename.empty() && ddPsfGridWidth == 1 &&
-      ddPsfGridHeight == 1) {
-    if (!facetSolutionFiles.empty())
+  const bool uses_dd_psf = !(ddPsfGridWidth == 1 && ddPsfGridHeight == 1);
+  const bool apply_facet_solutions = !facetSolutionFiles.empty();
+  if (facetRegionFilename.empty() && !uses_dd_psf) {
+    if (apply_facet_solutions)
       throw std::runtime_error(
           "A facet solution file can only be specified in conjunction with a "
           "facet regions file. Either remove -apply-facet-solutions from the "
@@ -104,6 +107,7 @@ void Settings::Validate() const {
           "regions file with -facet-regions, or specify a grid of dd-psfs with "
           "-dd-psf-grid.");
   } else {
+    // Either DD PSFs or facets are used in this branch.
     if (polarizations.size() > 1) {
       // -join-polarizations required in order to write the pb.fits images
       // in PrimaryBeam::CorrectImages
@@ -114,17 +118,23 @@ void Settings::Validate() const {
             "independently. Add -join-polarizations to the command line to "
             "apply the facet beam for multiple polarizations");
       }
-      // This condition might become a bit more specific once xx,yy polarization
-      // correction for h5 AND beam are implemented
-      if (applyFacetBeam && !facetSolutionFiles.empty() &&
-          !aocommon::Polarization::HasFullStokesPolarization(polarizations)) {
-        throw std::runtime_error(
-            "Applying h5parm solutions AND beam correction on multiple "
-            "polarizations is not yet supported.");
+    }
+
+    if (apply_facet_solutions || applyFacetBeam) {
+      if (polarizations !=
+          std::set{PolarizationEnum::XX, PolarizationEnum::YY}) {
+        for (PolarizationEnum p : polarizations) {
+          if (!Polarization::IsStokes(p)) {
+            throw std::runtime_error(
+                "A polarized facet correction was requested that is not "
+                "implemented: facet corrections can only be applied on Stokes "
+                "images or on the combination of XX and YY.");
+          }
+        }
       }
     }
 
-    if (!facetSolutionFiles.empty()) {
+    if (apply_facet_solutions) {
       if (facetSolutionFiles.size() != 1 &&
           facetSolutionFiles.size() != filenames.size()) {
         throw std::runtime_error(
@@ -158,6 +168,18 @@ void Settings::Validate() const {
         "Parameter -feather-size was specified without enabling facetting.");
   }
 
+  if (shared_facet_reads && writeImagingWeightSpectrumColumn) {
+    throw std::runtime_error(
+        "-store-imaging-weights is currently not possible to combine with "
+        "-shared-facet-reads.");
+  }
+
+  if (gridderType == GridderType::WTowers) {
+#ifndef BUILD_WTOWERS
+    throw std::runtime_error("WSClean not built with w-towers support");
+#endif
+  }
+
   if (gridderType == GridderType::IDG) {
     const bool stokesIOnly =
         polarizations.size() == 1 &&
@@ -218,6 +240,39 @@ void Settings::Validate() const {
     }
   }
 
+  if (gridderType == GridderType::FacetIDG) {
+    if (trimmedImageWidth != trimmedImageHeight) {
+      throw std::runtime_error(
+          "IDG can not yet make rectangular images -- this will be implemented "
+          "at a later time.");
+    }
+
+    if (parallelGridding != 1) {
+      throw std::runtime_error(
+          "Parallel gridding can not be combined with IDG");
+    }
+
+    if (baselineDependentAveragingInWavelengths != 0.0) {
+      throw std::runtime_error(
+          "IDG cannot be combined with baseline-dependent averaging. Please "
+          "remove baseline-averaging "
+          "option from your command.");
+    }
+
+    for (const auto& filename : filenames) {
+      casacore::MeasurementSet ms(filename);
+      const std::string& bda_factors = "BDA_FACTORS";
+      const bool has_bda = ms.keywordSet().isDefined(bda_factors) &&
+                           (ms.keywordSet().asTable(bda_factors).nrow() > 0);
+      if (has_bda) {
+        throw std::runtime_error(
+            "IDG cannot be combined with the baseline-dependently averaged "
+            "measurement set " +
+            filename);
+      }
+    }
+  }
+
   if (UseMpi()) {
     if (!masterDoesWork && nMpiNodes <= 1) {
       throw std::runtime_error(
@@ -292,6 +347,15 @@ void Settings::Validate() const {
     throw std::runtime_error(
         "Can not both force reordering and force not reordering!");
 
+  if (deconvolutionChannelCount > channelsOut) {
+    throw std::runtime_error("The '-deconvolution-channels' option is set to " +
+                             std::to_string(deconvolutionChannelCount) +
+                             ", whereas the number of output channels is " +
+                             std::to_string(channelsOut) +
+                             ". The deconvolution channels must be equal to or "
+                             "less than the number of output channels.");
+  }
+
   if (deconvolutionChannelCount != 0 &&
       deconvolutionChannelCount != channelsOut &&
       spectralFittingMode ==
@@ -304,6 +368,19 @@ void Settings::Validate() const {
         "full number of channels. The most useful and common spectral fitting "
         "function is -fit-spectral-pol.");
 
+  if (spectralFittingMode ==
+      schaapcommon::fitters::SpectralFittingMode::kRotationMeasure) {
+    if (polarizations != std::set{aocommon::Polarization::StokesQ,
+                                  aocommon::Polarization::StokesU}) {
+      throw std::runtime_error(
+          "RM fitting is only possible when imaging Stokes Q and U "
+          "polarizations");
+    }
+    if (!joinedPolarizationDeconvolution)
+      throw std::runtime_error(
+          "Joined polarization deconvolution must be enabled for RM fitting");
+  }
+
   if (savePsfPb && !(applyPrimaryBeam || gridWithBeam))
     throw std::runtime_error(
         "You can not save the primary-beam corrected PSF without enabling "
@@ -354,6 +431,12 @@ void Settings::Validate() const {
         "required joining or linking the polarizations.");
   }
 
+  if (UseFacetCorrections() && baselineDependentAveragingInWavelengths != 0.0) {
+    throw std::runtime_error(
+        "Baseline-dependent averaging can not be combined with facet "
+        "corrections.");
+  }
+
   if (reuseDirty && (gridWithBeam || !atermConfigFilename.empty())) {
     throw std::runtime_error(
         "Reusing dirty image and beam/aterm corrections"
@@ -381,7 +464,7 @@ void Settings::checkPolarizations() const {
           "'-join-polarizations' or disable cleaning.");
   }
 
-  for (aocommon::PolarizationEnum p : linkedPolarizations) {
+  for (PolarizationEnum p : linkedPolarizations) {
     if (polarizations.count(p) == 0) {
       std::ostringstream str;
       str << "Linked polarization cleaning was requested for polarization "
@@ -418,20 +501,6 @@ void Settings::checkPolarizations() const {
         "w-stacking gridder. Either add '-gridder wstacking' to the command "
         "line, or image a different set of polarizations (e.g. iquv).");
   }
-  if (UseFacetCorrections()) {
-    const bool is_i =
-        (polarizations == std::set{aocommon::PolarizationEnum::StokesI});
-    const bool is_xx_yy =
-        (polarizations == std::set{aocommon::PolarizationEnum::XX,
-                                   aocommon::PolarizationEnum::YY});
-    const bool is_iquv =
-        aocommon::Polarization::HasFullStokesPolarization(polarizations);
-    if (!(is_i || is_xx_yy || is_iquv)) {
-      throw std::runtime_error(
-          "Facet imaging with facet corrections is only possible for Stokes I, "
-          "XX+YY or Stokes IQUV imaging");
-    }
-  }
 }
 
 void Settings::Propagate(bool verbose) {
@@ -462,6 +531,17 @@ void Settings::Propagate(bool verbose) {
     doReorder = determineReorder();
     dataColumnName = determineDataColumn(verbose);
   }
+
+  // In the case of IDG we have to directly ask for all four polarizations.
+  request_polarizations_at_once =
+      gridderType == GridderType::IDG && polarizations.size() > 1;
+  // In case XY/YX polarizations are requested, we should not parallelize over
+  // those since they need to be combined after imaging, and this currently
+  // requires XY before YX.
+  const bool has_xy = polarizations.count(aocommon::Polarization::XY) != 0;
+  const bool has_yx = polarizations.count(aocommon::Polarization::YX) != 0;
+  parallelize_polarizations = !request_polarizations_at_once &&
+                              polarizations.size() > 1 && (!has_xy || !has_yx);
 }
 
 void Settings::RecalculateDerivedDimensions(bool verbose) {
@@ -513,6 +593,8 @@ radler::Settings Settings::GetRadlerSettings() const {
   radler_settings.thread_count = threadCount;
   radler_settings.prefix_name = prefixName;
   radler_settings.linked_polarizations = linkedPolarizations;
+  radler_settings.component_optimization_algorithm =
+      componentOptimizationAlgorithm;
   radler_settings.parallel.grid_width = parallelDeconvolutionGridWidth;
   radler_settings.parallel.grid_height = parallelDeconvolutionGridHeight;
   radler_settings.parallel.max_threads = parallelDeconvolutionMaxThreads;
@@ -521,8 +603,11 @@ radler::Settings Settings::GetRadlerSettings() const {
       absoluteDeconvolutionThreshold.value_or(0.0);
   radler_settings.auto_mask_sigma = autoMaskSigma;
   radler_settings.absolute_auto_mask_threshold = absoluteAutoMaskThreshold;
+  radler_settings.major_auto_mask_iteration_count = majorAutoMaskIterations;
   radler_settings.minor_loop_gain = deconvolutionGain;
   radler_settings.major_loop_gain = deconvolutionMGain;
+  radler_settings.initial_iteration_boost = deconvolutionBoosting;
+  radler_settings.major_iteration_strategy = majorIterationStrategy;
   radler_settings.local_rms.method = localRMSMethod;
   radler_settings.local_rms.strength = localRMSStrength;
   radler_settings.local_rms.window = localRMSWindow;
@@ -585,11 +670,10 @@ radler::Settings Settings::GetRadlerSettings() const {
   return radler_settings;
 }
 
-aocommon::PolarizationEnum Settings::GetProviderPolarization(
-    aocommon::PolarizationEnum entry_polarization) const {
+PolarizationEnum Settings::GetProviderPolarization(
+    PolarizationEnum entry_polarization) const {
   const bool xx_and_yy =
-      polarizations ==
-      std::set{aocommon::PolarizationEnum::XX, aocommon::PolarizationEnum::YY};
+      polarizations == std::set{PolarizationEnum::XX, PolarizationEnum::YY};
   const bool stokes_i =
       polarizations.size() == 1 &&
       (*polarizations.begin()) == aocommon::Polarization::StokesI;
@@ -611,9 +695,9 @@ aocommon::PolarizationEnum Settings::GetProviderPolarization(
     return aocommon::Polarization::Instrumental;
   } else {
     bool requires_instrumental = false;
-    for (aocommon::PolarizationEnum p : polarizations) {
+    for (PolarizationEnum p : polarizations) {
       if (aocommon::Polarization::IsStokes(p) &&
-          p != aocommon::PolarizationEnum::StokesI) {
+          p != PolarizationEnum::StokesI) {
         requires_instrumental = UseFacetCorrections();
         break;
       }
