@@ -15,6 +15,7 @@
 #include <aocommon/system.h>
 
 #include <schaapcommon/fitters/spectralfitter.h>
+#include <schaapcommon/reordering/msselection.h>
 #include <schaapcommon/reordering/storagemanagertype.h>
 
 #include <radler/settings.h>
@@ -22,7 +23,15 @@
 namespace wsclean {
 
 enum class DirectFTPrecision { Float, Double, LongDouble };
-enum class GridderType { WStacking, WGridder, TunedWGridder, DirectFT, IDG };
+enum class GridderType {
+  WStacking,
+  WGridder,
+  TunedWGridder,
+  WTowers,
+  DirectFT,
+  IDG,
+  FacetIDG
+};
 enum class VisibilityReadMode { kScalar, kDiagonal, kFull };
 
 /**
@@ -44,7 +53,8 @@ class Settings {
     ImagingMode,
     PredictMode,
     RestoreMode,
-    RestoreListMode
+    RestoreListMode,
+    DrawModelMode
   } mode = ImagingMode;
   GridderType gridderType = GridderType::WGridder;
   size_t paddedImageWidth = 0, paddedImageHeight = 0;
@@ -62,7 +72,10 @@ class Settings {
   std::string restoreModel, restoreInput, restoreOutput;
   double manualBeamMajorSize = 0.0, manualBeamMinorSize = 0.0;
   double manualBeamPA = 0.0;
-  bool fittedBeam = true, theoreticBeam = false, circularBeam = false;
+  bool fittedBeam = true;
+  bool theoreticBeam = false;
+  bool circularBeam = false;
+  bool fitBeamWithNegatives = true;
   double beamFittingBoxSize = 10.0;
   bool continuedRun = false;
   double memFraction = 1.0, absMemLimit = 0.0;
@@ -95,6 +108,8 @@ class Settings {
       schaapcommon::reordering::StorageManagerType::Default;
   std::set<aocommon::PolarizationEnum> polarizations{
       aocommon::Polarization::StokesI};
+  bool request_polarizations_at_once = false;
+  bool parallelize_polarizations = false;
   std::string facetRegionFilename;
   std::optional<size_t> featherSize;
   std::set<size_t> spectralWindows;
@@ -105,6 +120,7 @@ class Settings {
   bool minGridResolution = true;
   bool makePSF = false;
   bool makePSFOnly = false;
+  bool skipFinalIteration = false;
   bool isWeightImageSaved = false;
   bool isUVImageSaved = false;
   bool isDirtySaved = true;
@@ -118,6 +134,7 @@ class Settings {
   bool doReorder = true;
   bool reuseReorder = false;
   bool saveReorder = false;
+  bool inMemoryData = false;
   bool subtractModel = false, modelUpdateRequired = true, mfWeighting = false;
   size_t fullResOffset = 0, fullResWidth = 0, fullResPad = 0;
   std::string beamModel, beamMode = "default";
@@ -129,7 +146,14 @@ class Settings {
   size_t primaryBeamGridSize = 32, primaryBeamUpdateTime = 1800;
   size_t ddPsfGridHeight = 1, ddPsfGridWidth = 1;
   DirectFTPrecision directFTPrecision = DirectFTPrecision::Double;
-  double wgridderAccuracy = 1e-4;
+  double gridder_accuracy = 0.0f;
+  // TODO: W-towers parameters should be determined automatically from
+  // gridder accuracy. Will need systematic testing / optimisation
+  int wtowers_subgrid_size = 256;
+  int wtowers_support = 8;
+  int wtowers_w_support = 8;
+  double wtowers_padding = 1.5;
+  double wtowers_w_padding = 1.5;
   std::string atermConfigFilename;
   double atermKernelSize = 5.0;
   bool gridWithBeam = false;
@@ -139,6 +163,7 @@ class Settings {
   bool solutionDirectionsCheck = true;
   VisibilityReadMode visibilityReadMode = VisibilityReadMode::kFull;
   bool applyFacetBeam = false;
+  bool applyTimeFrequencySmearing = false;
   double facetBeamUpdateTime = 120.0;  // in seconds.
   bool saveATerms = false;
   enum IDGMode {
@@ -156,6 +181,7 @@ class Settings {
   std::string simulatedBaselineNoiseFilename;
   bool compound_tasks = false;
   bool shared_facet_reads = false;
+  bool shared_facet_writes = false;
 
   /** @{
    * These settings all relate to the deconvolution.
@@ -169,10 +195,17 @@ class Settings {
   std::optional<double> absoluteDeconvolutionThreshold;
   std::optional<double> autoMaskSigma;
   std::optional<double> absoluteAutoMaskThreshold;
-  double deconvolutionGain = 0.1, deconvolutionMGain = 1.0;
+  size_t majorAutoMaskIterations = 2;
+  double deconvolutionGain = 0.1;
+  double deconvolutionMGain = 1.0;
+  double deconvolutionBoosting = 1.0;
+  radler::MajorIterationStrategy majorIterationStrategy =
+      radler::MajorIterationStrategy::kDual;
   double localRMSWindow = 25.0;
   double localRMSStrength = 1.0;
   radler::LocalRmsMethod localRMSMethod = radler::LocalRmsMethod::kNone;
+  radler::OptimizationAlgorithm componentOptimizationAlgorithm =
+      radler::OptimizationAlgorithm::kClean;
   bool saveSourceList = false;
   size_t deconvolutionIterationCount = 0;
   size_t majorIterationCount = 12;
@@ -200,6 +233,14 @@ class Settings {
       schaapcommon::fitters::SpectralFittingMode::kNoFitting;
   size_t spectralFittingTerms = 0;
   std::string forcedSpectrumFilename;
+  std::string inputSkyModelFilename;
+  size_t sincWindowSize = 127;
+  size_t drawnSpectralTermCount = 1;
+  double drawnSkyModelFrequency = 0.0;
+  double drawnSkyModelBandwidth = 0.0;
+  bool useManualPhaseCentre = false;
+  double drawnPhaseCentreRA = 0.0;
+  double drawnPhaseCentreDec = 0.0;
   /**
    * The number of channels used during deconvolution. This can be used to
    * image with more channels than deconvolution. Before deconvolution,
