@@ -26,6 +26,7 @@
 
 #include "../math/renderer.h"
 #include "../math/tophatconvolution.h"
+#include "../math/subpixelrenderer.h"
 
 #include "../model/model.h"
 
@@ -49,6 +50,7 @@
 #include <iostream>
 #include <memory>
 
+using aocommon::CoordinateSystem;
 using aocommon::Image;
 using aocommon::Logger;
 using aocommon::Polarization;
@@ -284,8 +286,8 @@ void WSClean::processFullPSF(Image& image, const ImagingTableEntry& entry) {
   double initialFitSize =
       std::max(_infoPerChannel[channelIndex].beamSizeEstimate, minPixelScale);
   double bMaj, bMin, bPA, bTheoretical;
-  ImageOperations::DetermineBeamSize(settings, bMaj, bMin, bPA, bTheoretical,
-                                     image, initialFitSize);
+  math::DetermineBeamSize(settings, bMaj, bMin, bPA, bTheoretical, image,
+                          initialFitSize);
   // Create a temporary copy of the output channel info
   // and put the fitting and normalization result in the copy
   OutputChannelInfo channel_info(_infoPerChannel[channelIndex]);
@@ -642,14 +644,15 @@ void WSClean::RunClean() {
   _observationInfo = getObservationInfo();
   std::tie(_l_shift, _m_shift) = getLMShift();
 
+  const bool make_square = _settings.gridderType == GridderType::IDG ||
+                           _settings.gridderType == GridderType::FacetIDG;
   std::vector<std::shared_ptr<schaapcommon::facets::Facet>> facets =
       FacetReader::ReadFacets(
           _settings.facetRegionFilename, _settings.trimmedImageWidth,
           _settings.trimmedImageHeight, _settings.pixelScaleX,
           _settings.pixelScaleY, _observationInfo.phaseCentreRA,
           _observationInfo.phaseCentreDec, _l_shift, _m_shift,
-          _settings.imagePadding, _settings.gridderType == GridderType::IDG,
-          _settings.GetFeatherSize());
+          _settings.imagePadding, make_square, _settings.GetFeatherSize());
   _facetCount = facets.size();
 
   std::vector<std::shared_ptr<schaapcommon::facets::Facet>> dd_psfs;
@@ -659,8 +662,7 @@ void WSClean::RunClean() {
             _settings.trimmedImageWidth, _settings.trimmedImageHeight,
             _settings.pixelScaleX, _settings.pixelScaleY,
             _observationInfo.phaseCentreRA, _observationInfo.phaseCentreDec,
-            _l_shift, _m_shift, _settings.imagePadding,
-            _settings.gridderType == GridderType::IDG, 0);
+            _l_shift, _m_shift, _settings.imagePadding, make_square, 0);
     dd_psfs = CreateFacetGrid(facet_data, _settings.ddPsfGridWidth,
                               _settings.ddPsfGridHeight);
   }
@@ -715,15 +717,13 @@ void WSClean::RunClean() {
 
     _imageWeightCache = createWeightCache();
 
-    _image_weight_initializer = std::make_unique<ImageWeightInitializer>(
-        _settings, _globalSelection, _msBands,
-        _msHelper->GetReorderedMsHandles());
-    if (_settings.mfWeighting)
-      _image_weight_initializer->InitializeMf(_imagingTable,
-                                              *_imageWeightCache);
+    if (_settings.mfWeighting) {
+      InitializeMfWeights(_settings, _imagingTable, *_imageWeightCache,
+                          *_msHelper);
+    }
     _griddingTaskFactory = std::make_unique<GriddingTaskFactory>(
-        *_msHelper, *_image_weight_initializer, _observationInfo, _l_shift,
-        _m_shift, _imagingTable.EntryCount());
+        *_msHelper, _settings, _observationInfo, _l_shift, _m_shift,
+        _imagingTable.EntryCount());
     _griddingTaskManager = GriddingTaskManager::Make(_settings);
     std::unique_ptr<PrimaryBeam> primaryBeam;
     for (size_t groupIndex = 0;
@@ -734,7 +734,6 @@ void WSClean::RunClean() {
 
     _griddingTaskManager.reset();
     _griddingTaskFactory.reset();
-    _image_weight_initializer.reset();
     // Resetting the MsHelper will destroy its reordered ms handles and
     // thereby clear the temporary files if -save-reordered is not present.
     _msHelper.reset();
@@ -753,22 +752,21 @@ void WSClean::RunClean() {
             std::optional<size_t> dd_psf_dir =
                 _ddPsfCount ? std::optional<size_t>(direction) : std::nullopt;
             OutputChannelInfo mfs_info = _infoForMFS;
-            ImageOperations::MakeMFSImage(_settings, _infoPerChannel, mfs_info,
-                                          "psf", intervalIndex, pol,
-                                          ImageFilenameType::Psf, dd_psf_dir);
+            math::MakeMFSImage(_settings, _infoPerChannel, mfs_info, "psf",
+                               intervalIndex, pol, ImageFilenameType::Psf,
+                               dd_psf_dir);
             if (_settings.savePsfPb)
-              ImageOperations::MakeMFSImage(
-                  _settings, _infoPerChannel, mfs_info, "psf-pb", intervalIndex,
-                  pol, ImageFilenameType::Psf, dd_psf_dir);
+              math::MakeMFSImage(_settings, _infoPerChannel, mfs_info, "psf-pb",
+                                 intervalIndex, pol, ImageFilenameType::Psf,
+                                 dd_psf_dir);
             if (direction == _infoForMFS.centralPsfIndex || _ddPsfCount == 0) {
               _infoForMFS = mfs_info;
             }
           }
         }
         if (griddingUsesATerms()) {
-          ImageOperations::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
-                                        "", intervalIndex, pol,
-                                        ImageFilenameType::Beam);
+          math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS, "",
+                             intervalIndex, pol, ImageFilenameType::Beam);
           // When faceting without beam, no beam images are stored, so skip
           // making the MFS beams in this case. In all other cases with beam, do
           // make them:
@@ -779,10 +777,9 @@ void WSClean::RunClean() {
           constexpr size_t n_matrix_elements = 16;
           for (size_t beam_index = 0; beam_index != n_matrix_elements;
                ++beam_index) {
-            ImageOperations::MakeMFSImage(
-                _settings, _infoPerChannel, _infoForMFS,
-                std::to_string(beam_index), intervalIndex, pol,
-                ImageFilenameType::Beam);
+            math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                               std::to_string(beam_index), intervalIndex, pol,
+                               ImageFilenameType::Beam);
           }
         }
 
@@ -790,68 +787,65 @@ void WSClean::RunClean() {
               _settings.polarizations.count(Polarization::XY) != 0) &&
             !_settings.makePSFOnly) {
           if (_settings.isDirtySaved)
-            ImageOperations::MakeMFSImage(_settings, _infoPerChannel,
-                                          _infoForMFS, "dirty", intervalIndex,
-                                          pol, ImageFilenameType::Normal);
+            math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS, "dirty",
+                               intervalIndex, pol, ImageFilenameType::Normal);
           if (_settings.deconvolutionIterationCount == 0) {
-            ImageOperations::MakeMFSImage(_settings, _infoPerChannel,
-                                          _infoForMFS, "image", intervalIndex,
-                                          pol, ImageFilenameType::Normal);
+            math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS, "image",
+                               intervalIndex, pol, ImageFilenameType::Normal);
             if (usesBeam())
-              ImageOperations::MakeMFSImage(
-                  _settings, _infoPerChannel, _infoForMFS, "image-pb",
-                  intervalIndex, pol, ImageFilenameType::Normal);
+              math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                 "image-pb", intervalIndex, pol,
+                                 ImageFilenameType::Normal);
           } else {
-            ImageOperations::MakeMFSImage(
-                _settings, _infoPerChannel, _infoForMFS, "residual",
-                intervalIndex, pol, ImageFilenameType::Normal);
-            ImageOperations::MakeMFSImage(_settings, _infoPerChannel,
-                                          _infoForMFS, "model", intervalIndex,
-                                          pol, ImageFilenameType::Normal);
-            ImageOperations::RenderMFSImage(_settings, _infoForMFS,
-                                            intervalIndex, pol, false, false);
+            math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                               "residual", intervalIndex, pol,
+                               ImageFilenameType::Normal);
+            math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS, "model",
+                               intervalIndex, pol, ImageFilenameType::Normal);
+            math::RenderMFSImage(_settings, _infoForMFS, intervalIndex, pol,
+                                 false, false);
             if (usesBeam()) {
-              ImageOperations::MakeMFSImage(
-                  _settings, _infoPerChannel, _infoForMFS, "residual-pb",
-                  intervalIndex, pol, ImageFilenameType::Normal);
-              ImageOperations::MakeMFSImage(
-                  _settings, _infoPerChannel, _infoForMFS, "model-pb",
-                  intervalIndex, pol, ImageFilenameType::Normal);
-              ImageOperations::RenderMFSImage(_settings, _infoForMFS,
-                                              intervalIndex, pol, false, true);
+              math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                 "residual-pb", intervalIndex, pol,
+                                 ImageFilenameType::Normal);
+              math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                 "model-pb", intervalIndex, pol,
+                                 ImageFilenameType::Normal);
+              math::RenderMFSImage(_settings, _infoForMFS, intervalIndex, pol,
+                                   false, true);
             }
           }
           if (Polarization::IsComplex(pol)) {
             if (_settings.isDirtySaved)
-              ImageOperations::MakeMFSImage(_settings, _infoPerChannel,
-                                            _infoForMFS, "dirty", intervalIndex,
-                                            pol, ImageFilenameType::Imaginary);
+              math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                 "dirty", intervalIndex, pol,
+                                 ImageFilenameType::Imaginary);
             if (_settings.deconvolutionIterationCount == 0) {
-              ImageOperations::MakeMFSImage(_settings, _infoPerChannel,
-                                            _infoForMFS, "image", intervalIndex,
-                                            pol, ImageFilenameType::Imaginary);
+              math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                 "image", intervalIndex, pol,
+                                 ImageFilenameType::Imaginary);
               if (usesBeam())
-                ImageOperations::MakeMFSImage(
-                    _settings, _infoPerChannel, _infoForMFS, "image-pb",
-                    intervalIndex, pol, ImageFilenameType::Imaginary);
+                math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                   "image-pb", intervalIndex, pol,
+                                   ImageFilenameType::Imaginary);
             } else {
-              ImageOperations::MakeMFSImage(
-                  _settings, _infoPerChannel, _infoForMFS, "residual",
-                  intervalIndex, pol, ImageFilenameType::Imaginary);
-              ImageOperations::MakeMFSImage(_settings, _infoPerChannel,
-                                            _infoForMFS, "model", intervalIndex,
-                                            pol, ImageFilenameType::Imaginary);
-              ImageOperations::RenderMFSImage(_settings, _infoForMFS,
-                                              intervalIndex, pol, true, false);
+              math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                 "residual", intervalIndex, pol,
+                                 ImageFilenameType::Imaginary);
+              math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                 "model", intervalIndex, pol,
+                                 ImageFilenameType::Imaginary);
+              math::RenderMFSImage(_settings, _infoForMFS, intervalIndex, pol,
+                                   true, false);
               if (usesBeam()) {
-                ImageOperations::MakeMFSImage(
-                    _settings, _infoPerChannel, _infoForMFS, "residual-pb",
-                    intervalIndex, pol, ImageFilenameType::Imaginary);
-                ImageOperations::MakeMFSImage(
-                    _settings, _infoPerChannel, _infoForMFS, "model-pb",
-                    intervalIndex, pol, ImageFilenameType::Imaginary);
-                ImageOperations::RenderMFSImage(_settings, _infoForMFS,
-                                                intervalIndex, pol, true, true);
+                math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                   "residual-pb", intervalIndex, pol,
+                                   ImageFilenameType::Imaginary);
+                math::MakeMFSImage(_settings, _infoPerChannel, _infoForMFS,
+                                   "model-pb", intervalIndex, pol,
+                                   ImageFilenameType::Imaginary);
+                math::RenderMFSImage(_settings, _infoForMFS, intervalIndex, pol,
+                                     true, true);
               }
             }
           }
@@ -918,13 +912,9 @@ void WSClean::RunPredict() {
     else if (_settings.doReorder)
       _msHelper->PerformReordering(_imagingTable, true);
 
-    _image_weight_initializer = std::make_unique<ImageWeightInitializer>(
-        _settings, _globalSelection, _msBands,
-        _msHelper->GetReorderedMsHandles());
-
     _griddingTaskFactory = std::make_unique<GriddingTaskFactory>(
-        *_msHelper, *_image_weight_initializer, _observationInfo, _l_shift,
-        _m_shift, _imagingTable.EntryCount());
+        *_msHelper, _settings, _observationInfo, _l_shift, _m_shift,
+        _imagingTable.EntryCount());
 
     if (_facetCount != 0) {
       std::string prefix =
@@ -934,13 +924,14 @@ void WSClean::RunPredict() {
       aocommon::FitsReader reader(prefix + PredictModelFileSuffix());
       overrideImageSettings(reader);
       if (intervalIndex == 0) {
+        const bool make_square = _settings.gridderType == GridderType::IDG ||
+                                 _settings.gridderType == GridderType::FacetIDG;
         facets = FacetReader::ReadFacets(
             _settings.facetRegionFilename, _settings.trimmedImageWidth,
             _settings.trimmedImageHeight, _settings.pixelScaleX,
             _settings.pixelScaleY, _observationInfo.phaseCentreRA,
             _observationInfo.phaseCentreDec, _l_shift, _m_shift,
-            _settings.imagePadding, _settings.gridderType == GridderType::IDG,
-            _settings.GetFeatherSize());
+            _settings.imagePadding, make_square, _settings.GetFeatherSize());
 
         // FIXME: raise warning if facets do not cover the entire image, see
         // AST-429
@@ -955,6 +946,59 @@ void WSClean::RunPredict() {
   }
 }
 
+void WSClean::DrawModel() {
+  double phase_centre_ra = 0.0;
+  double phase_centre_dec = 0.0;
+  double l_shift = 0.0;
+  double m_shift = 0.0;
+
+  if (_settings.useManualPhaseCentre) {
+    phase_centre_ra = _settings.drawnPhaseCentreRA;
+    phase_centre_dec = _settings.drawnPhaseCentreDec;
+  } else if (_settings.filenames.size() != 0) {
+    _observationInfo = getObservationInfo();
+    phase_centre_ra = _observationInfo.phaseCentreRA;
+    phase_centre_dec = _observationInfo.phaseCentreDec;
+
+    std::tie(l_shift, m_shift) = getLMShift();
+  } else {
+    throw std::runtime_error("No phase centre is provided.");
+  }
+
+  CoordinateSystem cs;
+  cs.width = _settings.trimmedImageWidth;
+  cs.height = _settings.trimmedImageHeight;
+  cs.ra = phase_centre_ra;
+  cs.dec = phase_centre_dec;
+  cs.dl = _settings.pixelScaleX;
+  cs.dm = _settings.pixelScaleY;
+  cs.l_shift = l_shift;
+  cs.m_shift = m_shift;
+
+  fftwf_make_planner_thread_safe();
+
+  std::vector<Image> images = math::RenderSubPixelModel(
+      _settings.inputSkyModelFilename, cs, _settings.drawnSkyModelFrequency,
+      _settings.drawnSkyModelBandwidth, _settings.sincWindowSize,
+      _settings.drawnSpectralTermCount, _settings.memFraction,
+      _settings.absMemLimit);
+
+  for (size_t image_index = 0; image_index < images.size(); ++image_index) {
+    std::string fits_filename = _settings.prefixName;
+    fits_filename += "-term-" + std::to_string(image_index) + ".fits";
+
+    aocommon::FitsWriter fits_writer;
+    fits_writer.SetImageDimensions(cs.width, cs.height, cs.ra, cs.dec, cs.dl,
+                                   cs.dm);
+    fits_writer.SetPhaseCentreShift(cs.l_shift, cs.m_shift);
+    fits_writer.SetFrequency(_settings.drawnSkyModelFrequency,
+                             _settings.drawnSkyModelBandwidth);
+
+    Logger::Info << "Writing sky model image " << fits_filename << "...\n";
+    fits_writer.Write(fits_filename, images[image_index].Data());
+  }
+}
+
 double WSClean::minTheoreticalBeamSize(const ImagingTable& table) const {
   double beam = 0.0;
   for (const ImagingTableEntry& e : table) {
@@ -990,18 +1034,6 @@ void WSClean::runIndependentGroup(ImagingTable& groupTable,
                                  _settings.prefixName + "-matrix-beam");
   }
 
-  // In the case of IDG we have to directly ask for all four polarizations.
-  const bool requestPolarizationsAtOnce =
-      _settings.gridderType == GridderType::IDG &&
-      _settings.polarizations.size() > 1;
-
-  // In case XY/YX polarizations are requested, we should not parallelize over
-  // those since they need to be combined after imaging, and this currently
-  // requires XY before YX.
-  const bool parallelizePolarizations =
-      _settings.polarizations.count(Polarization::XY) == 0 &&
-      _settings.polarizations.count(Polarization::YX) == 0;
-
   _inversionWatch.Start();
   const bool doMakePSF = _settings.deconvolutionIterationCount > 0 ||
                          _settings.makePSF || _settings.makePSFOnly;
@@ -1028,20 +1060,19 @@ void WSClean::runIndependentGroup(ImagingTable& groupTable,
 
     _griddingTaskManager->Finish();
 
-    if (!doMakeDdPsf && !_settings.reusePsf)
-      stitchFacets(groupTable, _psfImages, false, true, false);
+    if (!doMakeDdPsf && !_settings.reusePsf) {
+      stitchFacets(groupTable, _psfImages, StitchImageType::kPSF);
+    }
   }
 
   if (!_settings.makePSFOnly) {
-    runFirstInversions(groupTable, primaryBeam, requestPolarizationsAtOnce,
-                       parallelizePolarizations);
+    runFirstInversions(groupTable, primaryBeam);
   }
 
   _inversionWatch.Pause();
 
   if (!_settings.makePSFOnly) {
-    runMajorIterations(groupTable, primaryBeam, requestPolarizationsAtOnce,
-                       parallelizePolarizations);
+    runMajorIterations(groupTable, primaryBeam);
   }
 
   Logger::Info << "Inversion: " << _inversionWatch.ToString()
@@ -1159,10 +1190,11 @@ void WSClean::writeFirstResidualImages(const ImagingTable& groupTable) const {
   }
 }
 
-void WSClean::writeModelImages(const ImagingTable& groupTable) const {
+void WSClean::WriteModelImages(const ImagingTable::Groups& facet_groups) const {
   Logger::Info << "Writing model image...\n";
   Image ptr(_settings.trimmedImageWidth, _settings.trimmedImageHeight);
-  for (const ImagingTableEntry& entry : groupTable) {
+  for (const ImagingTable::Group& group : facet_groups) {
+    const ImagingTableEntry& entry = *group.front();
     size_t ch = entry.outputChannelIndex;
     if (entry.polarization == Polarization::YX) {
       _modelImages.Load(ptr.Data(), Polarization::XY, ch, true);
@@ -1284,8 +1316,8 @@ void WSClean::readExistingModelImages(const ImagingTableEntry& entry,
       // because only now the image size and scale is known.
       _imageWeightCache = createWeightCache();
       if (_settings.mfWeighting)
-        _image_weight_initializer->InitializeMf(_imagingTable,
-                                                *_imageWeightCache);
+        InitializeMfWeights(_settings, _imagingTable, *_imageWeightCache,
+                            *_msHelper);
     }
 
     WSCFitsWriter writer(reader);
@@ -1364,7 +1396,7 @@ void WSClean::predictGroup(const ImagingTable& groupTable) {
       _settings.gridderType == GridderType::IDG &&
       _settings.polarizations.size() != 1;
 
-  resetModelColumns(groupTable.FacetGroups());
+  ResetModelColumnsIfUsingFacets(groupTable.FacetGroups());
   _griddingTaskManager->Start(getMaxNrMSProviders() *
                               (groupTable.MaxFacetGroupIndex() + 1));
   _predictingWatch.Start();
@@ -1409,9 +1441,10 @@ void WSClean::predictGroup(const ImagingTable& groupTable) {
                << ", cleaning: " << _deconvolutionWatch.ToString() << '\n';
 }
 
-void WSClean::resetModelColumns(const ImagingTable::Groups& facet_groups) {
+void WSClean::ResetModelColumnsIfUsingFacets(
+    const ImagingTable::Groups& facet_groups) {
   assert(!facet_groups.empty());
-  if (facet_groups.front().size() > 1) {
+  if (_facetCount != 0) {
     for (const ImagingTable::Group& facet_group : facet_groups) {
       resetModelColumns(*facet_group.front());
     }
@@ -1426,17 +1459,15 @@ void WSClean::resetModelColumns(const ImagingTableEntry& entry) {
 }
 
 void WSClean::runFirstInversions(ImagingTable& groupTable,
-                                 std::unique_ptr<PrimaryBeam>& primaryBeam,
-                                 bool requestPolarizationsAtOnce,
-                                 bool parallelizePolarizations) {
+                                 std::unique_ptr<PrimaryBeam>& primaryBeam) {
   std::vector<ImagingTable::Groups> facetGroupsQueue;
-  if (requestPolarizationsAtOnce) {
+  if (_settings.request_polarizations_at_once) {
     facetGroupsQueue.emplace_back(
         groupTable.FacetGroups([&](const ImagingTableEntry& entry) {
           return !entry.isDdPsf &&
                  (entry.polarization == *_settings.polarizations.begin());
         }));
-  } else if (parallelizePolarizations) {
+  } else if (_settings.parallelize_polarizations) {
     facetGroupsQueue.emplace_back(groupTable.FacetGroups());
   } else {
     // Only use parallelism for entries with the same polarization.
@@ -1454,12 +1485,11 @@ void WSClean::runFirstInversions(ImagingTable& groupTable,
     _griddingTaskManager->Finish();
   }
 
-  if (requestPolarizationsAtOnce) {
+  if (_settings.request_polarizations_at_once) {
     groupTable.AssignGridDataFromPolarization(*_settings.polarizations.begin());
   }
   if (!_settings.reuseDirty)
-    stitchFacets(groupTable, _residualImages, _settings.isDirtySaved, false,
-                 false);
+    stitchFacets(groupTable, _residualImages, StitchImageType::kDirty);
 }
 
 void WSClean::runFirstInversionGroup(
@@ -1478,8 +1508,7 @@ void WSClean::runFirstInversionGroup(
           !_settings.facetSolutionFiles.empty()) {
         std::vector<MsListItem> msList = _msHelper->InitializeMsList(*entry);
         std::shared_ptr<ImageWeights> weights =
-            _image_weight_initializer->Initialize(*entry, msList,
-                                                  *_imageWeightCache);
+            InitializeWeights(_settings, *entry, msList, *_imageWeightCache);
         primaryBeam = std::make_unique<PrimaryBeam>(_settings);
         for (MsListItem& item : msList)
           primaryBeam->AddMS(std::move(item.ms_description));
@@ -1512,9 +1541,7 @@ void WSClean::runFirstInversionGroup(
 }
 
 void WSClean::runMajorIterations(ImagingTable& groupTable,
-                                 std::unique_ptr<PrimaryBeam>& primaryBeam,
-                                 bool requestPolarizationsAtOnce,
-                                 bool parallelizePolarizations) {
+                                 std::unique_ptr<PrimaryBeam>& primaryBeam) {
   std::unique_ptr<radler::WorkTable> deconvolution_table =
       groupTable.CreateDeconvolutionTable(_settings.deconvolutionChannelCount,
                                           _psfImages, _modelImages,
@@ -1543,19 +1570,21 @@ void WSClean::runMajorIterations(ImagingTable& groupTable,
       if (_majorIterationNr == 1 && _settings.deconvolutionMGain != 1.0 &&
           _settings.isFirstResidualSaved)
         writeFirstResidualImages(tableWithoutDdPsf);
-      const bool isFinished = !reachedMajorThreshold;
-      if (isFinished) {
-        writeModelImages(tableWithoutDdPsf);
+
+      const bool is_finished = !reachedMajorThreshold;
+      if (is_finished) {
+        WriteModelImages(facetGroups);
       }
 
-      partitionModelIntoFacets(facetGroups, false);
-      if (_settings.deconvolutionMGain != 1.0) {
-        resetModelColumns(facetGroups);
+      const bool skip_iteration = is_finished && _settings.skipFinalIteration;
+      if (!skip_iteration) partitionModelIntoFacets(facetGroups, false);
+      if (_settings.deconvolutionMGain != 1.0 && !skip_iteration) {
+        ResetModelColumnsIfUsingFacets(facetGroups);
         _griddingTaskManager->Start(
             getMaxNrMSProviders() *
             (tableWithoutDdPsf.MaxFacetGroupIndex() + 1));
 
-        if (requestPolarizationsAtOnce) {
+        if (_settings.request_polarizations_at_once) {
           // Only request one polarization for each facet/channel.
           // The gridder will grid all polarizations.
           const aocommon::PolarizationEnum first_polarization =
@@ -1577,7 +1606,7 @@ void WSClean::runMajorIterations(ImagingTable& groupTable,
           }
           _griddingTaskManager->Finish();
           _inversionWatch.Pause();
-        } else if (parallelizePolarizations) {
+        } else if (_settings.parallelize_polarizations) {
           _predictingWatch.Start();
           for (const ImagingTable::Group& facetGroup : facetGroups) {
             Predict(facetGroup);
@@ -1616,7 +1645,8 @@ void WSClean::runMajorIterations(ImagingTable& groupTable,
           }
           _inversionWatch.Pause();
         }
-        stitchFacets(tableWithoutDdPsf, _residualImages, false, false, false);
+        stitchFacets(tableWithoutDdPsf, _residualImages,
+                     StitchImageType::kOther);
       }
 
       ++_majorIterationNr;
@@ -1630,7 +1660,8 @@ void WSClean::runMajorIterations(ImagingTable& groupTable,
     // The model facet images have already been corrected for their average gain
     // correction, so the full image can just be re-stitch from the facet images
     // to make the "fpb" facet corrected model images.
-    stitchFacets(tableWithoutDdPsf, _modelImages, false, false, true);
+    stitchFacets(tableWithoutDdPsf, _modelImages,
+                 StitchImageType::kFacetPbModel);
   }
 
   for (const ImagingTable::Group& facetGroup : facetGroups) {
@@ -1757,19 +1788,50 @@ void WSClean::ApplyFacetCorrectionForSingleChannel(
                                   .averageFacetCorrection[facet_index]
                                   .GetMatrixValue();
     if (matrix.Invert()) {
-      if (facet_images.size() == 4) {
+      if (Polarization::HasFullStokesPolarization(_settings.polarizations)) {
         std::array<Image*, 4> images{&facet_images[0], &facet_images[1],
                                      &facet_images[2], &facet_images[3]};
         math::CorrectImagesForMuellerMatrix(matrix * stokes_i_sqrt, images);
-      } else if (facet_images.size() == 2) {
+      } else if (_settings.polarizations ==
+                 std::set{PolarizationEnum::XX, PolarizationEnum::YY}) {
         std::array<Image*, 2> images{&facet_images[0], &facet_images[1]};
         math::CorrectDualImagesForMuellerMatrix(matrix * stokes_i_sqrt, images);
       } else {
-        throw std::runtime_error(
-            "A polarized facet correction was requested that is not "
-            "implemented");
+        Logger::Warn << "Warning: Performing facet corrections with incomplete "
+                        "Stokes images. This may not\n"
+                        "correct for polarization leakage well.\n";
+
+        for (PolarizationEnum p : _settings.polarizations) {
+          if (!Polarization::IsStokes(p)) {
+            throw std::runtime_error(
+                "A polarized facet correction was requested that is not "
+                "implemented: facet corrections can only be applied on Stokes "
+                "images or on the combination of XX and YY.");
+          }
+        }
+
+        std::array<Image*, 4> images{nullptr, nullptr, nullptr, nullptr};
+        std::vector<Image>::iterator facet_image_iterator =
+            facet_images.begin();
+        for (PolarizationEnum p : _settings.polarizations) {
+          const size_t pol_index = Polarization::StokesToIndex(p);
+          images[pol_index] = &*facet_image_iterator;
+          ++facet_image_iterator;
+        }
+
+        Image zero_images[4];
+        for (size_t i = 0; i != images.size(); ++i) {
+          if (!images[i]) {
+            zero_images[i] = Image(facet_images.front().Width(),
+                                   facet_images.front().Height(), 0.0f);
+            images[i] = &zero_images[i];
+          }
+        }
+        math::CorrectImagesForMuellerMatrix(matrix * stokes_i_sqrt, images);
       }
     } else {
+      Logger::Warn << "Warning: Facet correction matrix is singular: can't be "
+                      "inverted.\n";
       for (Image& image : facet_images)
         image = std::numeric_limits<float>::quiet_NaN();
     }
@@ -1788,8 +1850,7 @@ void WSClean::ApplyFacetCorrectionForSingleChannel(
 }
 
 void WSClean::stitchFacets(const ImagingTable& table,
-                           CachedImageSet& image_cache, bool write_dirty,
-                           bool is_psf, bool is_facet_pb_model) {
+                           CachedImageSet& image_cache, StitchImageType type) {
   if (_facetCount != 0) {
     Logger::Info << "Stitching facets onto full image...\n";
     // Allocate full image
@@ -1800,8 +1861,9 @@ void WSClean::stitchFacets(const ImagingTable& table,
     for (size_t sq_group = 0; sq_group != table.SquaredGroupCount();
          ++sq_group) {
       const ImagingTable squared_group = table.GetSquaredGroup(sq_group);
-      const bool apply_correction =
-          !is_psf && !is_facet_pb_model && _settings.UseFacetCorrections();
+      const bool apply_correction = type != StitchImageType::kPSF &&
+                                    type != StitchImageType::kFacetPbModel &&
+                                    _settings.UseFacetCorrections();
       // If a matrix correction is needed, it is done before stitching. If a
       // scalar correction is needed, it's done "during" stitching to save time.
       const bool apply_matrix = _settings.polarizations.size() != 1;
@@ -1816,16 +1878,16 @@ void WSClean::stitchFacets(const ImagingTable& table,
       for (const ImagingTable::Group& facet_group :
            squared_group.FacetGroups()) {
         // The PSF is only once imaged for all polarizations
-        if (!is_psf || facet_group.front()->polarization ==
-                           *_settings.polarizations.begin()) {
+        if (type != StitchImageType::kPSF ||
+            facet_group.front()->polarization ==
+                *_settings.polarizations.begin()) {
           const size_t image_count = facet_group.front()->imageCount;
           for (size_t image_index = 0; image_index != image_count;
                ++image_index) {
-            stitchSingleGroup(facet_group, image_index, image_cache,
-                              write_dirty, is_psf, full_image, weight_image,
-                              facet_image, table.MaxFacetGroupIndex(),
-                              apply_correction && !apply_matrix,
-                              is_facet_pb_model);
+            stitchSingleGroup(facet_group, image_index, image_cache, type,
+                              full_image, weight_image, facet_image,
+                              table.MaxFacetGroupIndex(),
+                              apply_correction && !apply_matrix);
           }
         }
       }
@@ -1835,13 +1897,12 @@ void WSClean::stitchFacets(const ImagingTable& table,
 
 void WSClean::stitchSingleGroup(const ImagingTable::Group& facetGroup,
                                 size_t imageIndex, CachedImageSet& imageCache,
-                                bool writeDirty, bool isPSF, Image& fullImage,
+                                StitchImageType type, Image& fullImage,
                                 std::unique_ptr<Image>& weight_image,
                                 schaapcommon::facets::FacetImage& facetImage,
-                                size_t maxFacetGroupIndex, bool apply_scalar,
-                                bool is_facet_pb_model) {
+                                size_t maxFacetGroupIndex, bool apply_scalar) {
   const size_t feather_size =
-      is_facet_pb_model ? 0 : _settings.GetFeatherSize();
+      (type == StitchImageType::kFacetPbModel) ? 0 : _settings.GetFeatherSize();
   const bool isImaginary = (imageIndex == 1);
   fullImage = 0.0f;
   if (feather_size != 0) {
@@ -1908,29 +1969,31 @@ void WSClean::stitchSingleGroup(const ImagingTable::Group& facetGroup,
         fullImage[i] = 0.0;
     }
   }
-  if (writeDirty) {
+  if (type == StitchImageType::kDirty) {
     initializeModelImages(*facetGroup.front(), facetGroup.front()->polarization,
                           maxFacetGroupIndex);
     _residualImages.SetWSCFitsWriter(
         createWSCFitsWriter(*facetGroup.front(), false, false));
-    WSCFitsWriter writer(
-        createWSCFitsWriter(*facetGroup.front(), isImaginary, false));
-    Logger::Info << "Writing dirty image...\n";
-    writer.WriteImage("dirty.fits", fullImage);
+    if (_settings.isDirtySaved) {
+      WSCFitsWriter writer(
+          createWSCFitsWriter(*facetGroup.front(), isImaginary, false));
+      Logger::Info << "Writing dirty image...\n";
+      writer.WriteImage("dirty.fits", fullImage);
+    }
   }
-  if (is_facet_pb_model) {
+  if (type == StitchImageType::kFacetPbModel) {
     WSCFitsWriter writer(
         createWSCFitsWriter(*facetGroup.front(), isImaginary, false));
     Logger::Info << "Writing facet-corrected primary-beam model image...\n";
     writer.WriteImage("model-fpb.fits", fullImage);
   }
 
-  if (isPSF) {
+  if (type == StitchImageType::kPSF) {
     const ImagingTableEntry& entry = *facetGroup.front();
     processFullPSF(fullImage, entry);
   }
 
-  if (!is_facet_pb_model) {
+  if (type != StitchImageType::kFacetPbModel) {
     const size_t channelIndex = facetGroup.front()->outputChannelIndex;
     const PolarizationEnum polarization = facetGroup.front()->polarization;
     imageCache.Store(fullImage, polarization, channelIndex, isImaginary);
@@ -1944,8 +2007,8 @@ void WSClean::makeImagingTable(size_t outputIntervalIndex) {
     casacore::MeasurementSet ms(_settings.filenames[i]);
     _msBands[i] = aocommon::MultiBandData(ms);
     std::set<size_t> dataDescIds = _msBands[i].GetUsedDataDescIds(ms);
-    if (dataDescIds.size() != _msBands[i].DataDescCount()) {
-      Logger::Debug << dataDescIds.size() << "/" << _msBands[i].DataDescCount()
+    if (dataDescIds.size() != _msBands[i].BandCount()) {
+      Logger::Debug << dataDescIds.size() << "/" << _msBands[i].BandCount()
                     << " spws are used of " << _settings.filenames[i] << '\n';
     }
 
@@ -1995,8 +2058,8 @@ void WSClean::makeImagingTable(size_t outputIntervalIndex) {
         << channelSet.size() << " unique channels.";
     throw std::runtime_error(str.str());
   }
-  std::vector<aocommon::ChannelInfo> inputChannelFrequencies(channelSet.begin(),
-                                                             channelSet.end());
+  const std::vector<aocommon::ChannelInfo> inputChannelFrequencies(
+      channelSet.begin(), channelSet.end());
   Logger::Debug << "Total nr of channels found in measurement sets: "
                 << inputChannelFrequencies.size() << '\n';
 
@@ -2097,10 +2160,7 @@ void WSClean::makeImagingTableEntry(
                     << '\n';
   }
 
-  entry.msData.resize(_settings.filenames.size());
-  for (size_t msIndex = 0; msIndex != _settings.filenames.size(); ++msIndex) {
-    entry.msData[msIndex].bands.resize(_msBands[msIndex].DataDescCount());
-  }
+  entry.part_index_per_ms.resize(_settings.filenames.size());
 }
 
 void WSClean::makeImagingTableEntryChannelSettings(
