@@ -30,7 +30,6 @@
 
 #include "../msproviders/reorderedmsprovider.h"
 
-#include "imageweightinitializer.h"
 #include "mshelper.h"
 #include "stopwatch.h"
 #include "settings.h"
@@ -44,6 +43,8 @@ namespace wsclean {
 class ImageWeightCache;
 class PrimaryBeam;
 
+enum class StitchImageType { kPSF, kDirty, kFacetPbModel, kOther };
+
 class WSClean {
  public:
   WSClean();
@@ -65,6 +66,11 @@ class WSClean {
    */
   void RunPredict();
 
+  /**
+   * @brief Create a FITS image based on a skymodel.
+   */
+  void DrawModel();
+
  private:
   void runIndependentGroup(ImagingTable& groupTable,
                            std::unique_ptr<PrimaryBeam>& primaryBeam);
@@ -74,9 +80,7 @@ class WSClean {
   void predictGroup(const ImagingTable& groupTable);
 
   void runFirstInversions(ImagingTable& groupTable,
-                          std::unique_ptr<PrimaryBeam>& primaryBeam,
-                          bool requestPolarizationsAtOnce,
-                          bool parallelizePolarizations);
+                          std::unique_ptr<PrimaryBeam>& primaryBeam);
   /**
    * @brief Run first inversion on all entries within a group.
    * @details A group should contain all facets of a single image.
@@ -84,9 +88,7 @@ class WSClean {
   void runFirstInversionGroup(ImagingTable::Group& facetGroup,
                               std::unique_ptr<PrimaryBeam>& primaryBeam);
   void runMajorIterations(ImagingTable& groupTable,
-                          std::unique_ptr<PrimaryBeam>& primaryBeam,
-                          bool requestPolarizationsAtOnce,
-                          bool parallelizePolarizations);
+                          std::unique_ptr<PrimaryBeam>& primaryBeam);
 
   /**
    * Returns true when gridding is done with a-terms. This can either
@@ -118,7 +120,7 @@ class WSClean {
   ObservationInfo getObservationInfo() const;
   std::pair<double, double> getLMShift() const;
 
-  void resetModelColumns(const ImagingTable::Groups& facet_groups);
+  void ResetModelColumnsIfUsingFacets(const ImagingTable::Groups& facet_groups);
   void resetModelColumns(const ImagingTableEntry& entry);
   void storeAndCombineXYandYX(CachedImageSet& dest, size_t joinedChannelIndex,
                               const ImagingTableEntry& entry,
@@ -189,7 +191,7 @@ class WSClean {
    * @brief Stitch facets for all FacetGroups
    */
   void stitchFacets(const ImagingTable& table, CachedImageSet& image_cache,
-                    bool write_dirty, bool is_psf, bool is_facet_pb_model);
+                    StitchImageType type);
 
   /**
    * Stitch facet for a single (Facet)Group
@@ -199,12 +201,10 @@ class WSClean {
    */
   void stitchSingleGroup(const ImagingTable::Group& facetGroup,
                          size_t imageIndex, CachedImageSet& imageCache,
-                         bool writeDirty, bool isPSF,
-                         aocommon::Image& fullImage,
+                         StitchImageType type, aocommon::Image& fullImage,
                          std::unique_ptr<aocommon::Image>& weight_image,
                          schaapcommon::facets::FacetImage& facetImage,
-                         size_t maxFacetGroupIndex, bool apply_scalar,
-                         bool is_facet_pb_model);
+                         size_t maxFacetGroupIndex, bool apply_scalar);
   /**
    * Partition model image into facets and save them into fits files
    */
@@ -221,7 +221,7 @@ class WSClean {
                             bool isPredictOnly);
 
   void writeFirstResidualImages(const ImagingTable& groupTable) const;
-  void writeModelImages(const ImagingTable& groupTable) const;
+  void WriteModelImages(const ImagingTable::Groups& facet_groups) const;
 
   double minTheoreticalBeamSize(const ImagingTable& table) const;
 
@@ -259,7 +259,8 @@ class WSClean {
    */
   size_t getMaxNrMSProviders() const {
     size_t msCount = 0;
-    for (const auto& msBand : _msBands) msCount += msBand.DataDescCount();
+    for (const auto& msBand : _msBands)
+      msCount += msBand.HighestDataDescId() + 1;
     return msCount;
   }
 
@@ -272,7 +273,6 @@ class WSClean {
   OutputChannelInfo _infoForMFS;
 
   std::unique_ptr<MsHelper> _msHelper;
-  std::unique_ptr<ImageWeightInitializer> _image_weight_initializer;
   std::unique_ptr<GriddingTaskFactory> _griddingTaskFactory;
   std::unique_ptr<GriddingTaskManager> _griddingTaskManager;
   std::unique_ptr<ImageWeightCache> _imageWeightCache;
