@@ -39,34 +39,22 @@ void CorrectDualImagesForMuellerMatrix(
     const aocommon::HMC4x4& mueller_correction,
     std::array<aocommon::Image*, 2>& images);
 
+void DetermineBeamSize(const Settings& settings, double& bMaj, double& bMin,
+                       double& bPA, double& bTheoretical,
+                       const aocommon::Image& image, double initialEstimate);
+
+void MakeMFSImage(const Settings& settings,
+                  const std::vector<OutputChannelInfo>& infoPerChannel,
+                  OutputChannelInfo& mfsInfo, const std::string& suffix,
+                  size_t intervalIndex, aocommon::PolarizationEnum pol,
+                  ImageFilenameType image_type,
+                  std::optional<size_t> directionIndex = std::nullopt);
+
+void RenderMFSImage(const Settings& settings, const OutputChannelInfo& mfsInfo,
+                    size_t intervalIndex, aocommon::PolarizationEnum pol,
+                    bool isImaginary, bool isPBCorrected);
+
 }  // namespace math
-
-class ImageOperations {
- public:
-  static void FitBeamSize(const Settings& settings, double& bMaj, double& bMin,
-                          double& bPA, const aocommon::Image& image,
-                          double beamEstimate);
-
-  static void DetermineBeamSize(const Settings& settings, double& bMaj,
-                                double& bMin, double& bPA, double& bTheoretical,
-                                const aocommon::Image& image,
-                                double initialEstimate);
-
-  static void MakeMFSImage(const Settings& settings,
-                           const std::vector<OutputChannelInfo>& infoPerChannel,
-                           OutputChannelInfo& mfsInfo,
-                           const std::string& suffix, size_t intervalIndex,
-                           aocommon::PolarizationEnum pol,
-                           ImageFilenameType image_type,
-                           std::optional<size_t> directionIndex = std::nullopt);
-
-  static void RenderMFSImage(const Settings& settings,
-                             const OutputChannelInfo& mfsInfo,
-                             size_t intervalIndex,
-                             aocommon::PolarizationEnum pol, bool isImaginary,
-                             bool isPBCorrected);
-};
-
 }  // namespace wsclean
 
 #endif
