@@ -1,5 +1,6 @@
 #include "contiguousms.h"
-#include "msreaders/contiguousmsreader.h"
+
+#include <memory>
 
 #include <aocommon/logger.h>
 
@@ -9,42 +10,55 @@
 #include <casacore/measures/TableMeasures/ScalarMeasColumn.h>
 #include <casacore/tables/Tables/TableLocker.h>
 
-#include <memory>
+#include "operations.h"
+#include "msreaders/contiguousmsreader.h"
 
+using schaapcommon::reordering::ChannelRange;
+using schaapcommon::reordering::ContainsDataDescId;
 using schaapcommon::reordering::MSSelection;
 using schaapcommon::reordering::StorageManagerType;
 
 namespace wsclean {
+namespace {
+void OutputOpeningMessage(
+    const std::string& ms_path,
+    const aocommon::VectorMap<ChannelRange>& channel_ranges) {
+  std::vector<size_t> ids;
+  for (const ChannelRange& range : channel_ranges) {
+    if (!range.Empty()) ids.emplace_back(range.data_desc_id);
+  }
+  aocommon::Logger::Info << "Opening " << ms_path << ", ";
+  if (ids.size() == 1) {
+    aocommon::Logger::Info << "spw " << ids.front();
+  } else {
+    aocommon::Logger::Info << "multiple spws";
+  }
+  aocommon::Logger::Info << " with contiguous MS reader.\n";
+}
+}  // namespace
 
-ContiguousMS::ContiguousMS(const string& msPath,
-                           const std::string& dataColumnName,
-                           const std::string& modelColumnName,
-                           StorageManagerType modelStorageManager,
-                           const MSSelection& selection,
-                           aocommon::PolarizationEnum outputPolarization,
-                           size_t dataDescId, bool useMPI)
-    : _currentOutputRow(0),
-      _currentOutputTimestep(0),
-      _currentOutputTime(0.0),
-      _dataDescId(dataDescId),
-      _useMPI(useMPI),
-      _nAntenna(0),
-      _isModelColumnPrepared(false),
-      _selection(selection),
+ContiguousMS::ContiguousMS(
+    const string& msPath, const std::string& dataColumnName,
+    const std::string& modelColumnName, StorageManagerType modelStorageManager,
+    const MSSelection& selection,
+    const aocommon::VectorMap<ChannelRange>& channel_ranges,
+    aocommon::PolarizationEnum outputPolarization, bool useMPI)
+    : _useMPI(useMPI),
+      selection_(selection),
+      channel_ranges_(channel_ranges),
       _outputPolarization(outputPolarization),
       _msPath(msPath),
       _dataColumnName(dataColumnName),
       _modelColumnName(modelColumnName),
       _modelStorageManager(modelStorageManager) {
+  OutputOpeningMessage(_msPath, channel_ranges_);
   open();
 }
 
 void ContiguousMS::open() {
-  aocommon::Logger::Info << "Opening " << _msPath << ", spw " << _dataDescId
-                         << " with contiguous MS reader.\n";
-
   _ms = SynchronizedMS(_msPath, _useMPI ? casacore::TableLock::UserNoReadLocking
                                         : casacore::TableLock::DefaultLocking);
+  has_frequency_bda_ = wsclean::HasFrequencyBda(*_ms);
 
   _antenna1Column = casacore::ScalarColumn<int>(
       *_ms, casacore::MS::columnName(casacore::MSMainEnums::ANTENNA1));
@@ -62,22 +76,27 @@ void ContiguousMS::open() {
   _flagColumn = casacore::ArrayColumn<bool>(
       *_ms, casacore::MS::columnName(casacore::MSMainEnums::FLAG));
 
-  _inputPolarizations = GetMSPolarizations(_dataDescId, *_ms);
-
   const casacore::IPosition shape(_dataColumn.shape(0));
   _dataArray = casacore::Array<std::complex<float>>(shape);
   _weightSpectrumArray = casacore::Array<float>(shape);
   _imagingWeightSpectrumArray = casacore::Array<float>(shape);
   _flagArray = casacore::Array<bool>(shape);
-  _bandData = aocommon::MultiBandData(*_ms);
-
-  if (_bandData.BandCount() > 1) {
+  original_bands_ = aocommon::MultiBandData(*_ms);
+  selected_bands_ = MakeSelectedPartBands(original_bands_, channel_ranges_);
+  if (original_bands_.BandCount() > 1) {
     throw std::runtime_error(
         "This set contains multiple spws, and can therefore not be opened "
         "directly due to possible synchronization issues between spws. You can "
         "force reordering of the measurement by adding -reorder to the command "
         "line.");
   }
+
+  for (const ChannelRange& range : channel_ranges_) {
+    const size_t data_desc_id = range.data_desc_id;
+    input_polarizations_.AlwaysEmplace(data_desc_id,
+                                       GetMSPolarizations(data_desc_id, *_ms));
+  }
+
   _nAntenna = _ms->antenna().nrow();
 
   _msHasWeightSpectrum = OpenWeightSpectrumColumn(*_ms, _weightSpectrumColumn);
@@ -88,8 +107,7 @@ void ContiguousMS::open() {
         *_ms, casacore::MS::columnName(casacore::MSMainEnums::WEIGHT)));
   }
 
-  GetRowRangeAndIDMap(*_ms, _selection, _startRow, _endRow,
-                      std::set<size_t>{size_t(_dataDescId)}, _idToMSRow);
+  GetRowRange(*_ms, selection_, _startRow, _endRow);
   ResetWritePosition();
 }
 
@@ -102,10 +120,11 @@ void ContiguousMS::ResetWritePosition() {
   _currentOutputRow = _startRow - 1;
   _currentOutputTime = 0.0;
   // TODO: something similar needed in the ContiguousMSReader class?
-  if (_selection.HasInterval())
-    _currentOutputTimestep = _selection.IntervalStart() - 1;
-  else
+  if (selection_.HasInterval()) {
+    _currentOutputTimestep = selection_.IntervalStart() - 1;
+  } else {
     _currentOutputTimestep = -1;
+  }
   NextOutputRow();
 }
 
@@ -125,9 +144,9 @@ void ContiguousMS::NextOutputRow() {
       ++_currentOutputTimestep;
       _currentOutputTime = _timeColumn(_currentOutputRow);
     }
-  } while (!_selection.IsSelected(fieldId, _currentOutputTimestep, a1, a2,
+  } while (!selection_.IsSelected(fieldId, _currentOutputTimestep, a1, a2,
                                   uvw.data()) ||
-           (dataDescId != _dataDescId));
+           !ContainsDataDescId(channel_ranges_, dataDescId));
 }
 
 double ContiguousMS::StartTime() {
@@ -137,26 +156,17 @@ double ContiguousMS::StartTime() {
       .get();
 }
 
-size_t ContiguousMS::NChannels() {
-  if (_selection.HasChannelRange())
-    return _selection.ChannelRangeEnd() - _selection.ChannelRangeStart();
-  else
-    return _bandData[_dataDescId].ChannelCount();
+size_t ContiguousMS::NMaxChannels() {
+  return selected_bands_.MaxBandChannels();
 }
 
 size_t ContiguousMS::NPolarizations() {
-  switch (_outputPolarization) {
-    case aocommon::Polarization::Instrumental:
-      return 4;
-    case aocommon::Polarization::DiagonalInstrumental:
-      return 2;
-    default:
-      return 1;
-  }
+  return aocommon::Polarization::GetVisibilityCount(_outputPolarization);
 }
 
 void ContiguousMS::prepareModelColumn() {
-  InitializeModelColumn(*_ms, _modelColumnName, _modelStorageManager);
+  InitializeModelColumn(*_ms, _dataColumnName, _modelColumnName,
+                        _modelStorageManager);
   _modelColumn =
       casacore::ArrayColumn<casacore::Complex>(*_ms, _modelColumnName);
   const casacore::IPosition shape(_modelColumn.shape(0));
@@ -178,30 +188,38 @@ void ContiguousMS::WriteModel(const std::complex<float>* buffer, bool addToMS) {
 
   if (!_isModelColumnPrepared) prepareModelColumn();
 
-  size_t startChannel, endChannel;
-  if (_selection.HasChannelRange()) {
-    startChannel = _selection.ChannelRangeStart();
-    endChannel = _selection.ChannelRangeEnd();
-  } else {
-    startChannel = 0;
-    endChannel = _bandData[_dataDescId].ChannelCount();
-  }
+  const size_t data_desc_id = _dataDescIdColumn(_currentOutputRow);
+  const size_t start_channel = channel_ranges_[data_desc_id].start;
+  const size_t end_channel = channel_ranges_[data_desc_id].end;
 
   _modelColumn.get(_currentOutputRow, _modelArray);
   if (addToMS) {
-    schaapcommon::reordering::StoreData<true>(_modelArray.data(), startChannel,
-                                              endChannel, _inputPolarizations,
-                                              buffer, _outputPolarization);
+    schaapcommon::reordering::StoreData<true>(
+        _modelArray.data(), start_channel, end_channel,
+        input_polarizations_[data_desc_id], buffer, _outputPolarization);
   } else {
-    schaapcommon::reordering::StoreData<false>(_modelArray.data(), startChannel,
-                                               endChannel, _inputPolarizations,
-                                               buffer, _outputPolarization);
+    schaapcommon::reordering::StoreData<false>(
+        _modelArray.data(), start_channel, end_channel,
+        input_polarizations_[data_desc_id], buffer, _outputPolarization);
   }
   _modelColumn.put(_currentOutputRow, _modelArray);
 }
 
-void ContiguousMS::MakeIdToMSRowMapping(std::vector<size_t>& idToMSRow) {
-  idToMSRow = _idToMSRow;
+bool ContiguousMS::IsRegular() const {
+  auto IsNotEmpty = [](const ChannelRange& range) { return !range.Empty(); };
+
+  const size_t n_data_desc_ids = channel_ranges_.CountKeysIf(IsNotEmpty);
+  return n_data_desc_ids <= 1;
+}
+
+double ContiguousMS::Interval() { return GetMsInterval(*_ms); }
+
+ObservationInfo ContiguousMS::GetObservationInfo() {
+  return ReadObservationInfo(*_ms, selection_.FieldIds().front());
+}
+
+std::vector<std::string> ContiguousMS::GetAntennaNames() {
+  return wsclean::GetAntennaNames(_ms->antenna());
 }
 
 }  // namespace wsclean
