@@ -19,9 +19,7 @@ class ContiguousMSReader final : public MSReader {
 
   void NextInputRow() override;
 
-  void ReadMeta(double& u, double& v, double& w) override;
-
-  void ReadMeta(MSProvider::MetaData& metaData) override;
+  void ReadMeta(MSProvider::MetaData& metadata) override;
 
   void ReadData(std::complex<float>* buffer) override;
 
