@@ -21,6 +21,9 @@
 #include <optional>
 #include <stdexcept>
 
+#include <casacore/measures/Measures/MEpoch.h>
+#include <casacore/measures/TableMeasures/ScalarMeasColumn.h>
+
 #ifdef HAVE_EVERYBEAM
 #include <EveryBeam/aterms/atermconfig.h>
 #include <EveryBeam/griddedresponse/griddedresponse.h>
@@ -314,7 +317,27 @@ void PrimaryBeam::CorrectBeamForFacetGain(
 void PrimaryBeam::CorrectImages(aocommon::FitsWriter& writer,
                                 const ImageFilename& image_name,
                                 const std::string& filename_kind) {
-  if (settings_.polarizations.size() == 1 || filename_kind == "psf") {
+  // In the future, this condition could potentially also include IDG to allow
+  // making image sets other than only I or IQUV with IDG.
+  const bool use_squared_beam = settings_.UseFacetCorrections();
+  if (use_squared_beam) {
+    // This creates four images of which only one is used, so this can be
+    // made more efficient with some refactoring.
+    const PrimaryBeamImageSet beam_images = LoadStokesI(image_name);
+    for (PolarizationEnum polarization : settings_.polarizations) {
+      ImageFilename name(image_name);
+      name.SetPolarization(polarization);
+      aocommon::FitsReader reader(name.GetPrefix(settings_) + "-" +
+                                  filename_kind + ".fits");
+      Image image(reader.ImageWidth(), reader.ImageHeight());
+      reader.Read(image.Data());
+
+      beam_images.ApplyStokesI(image.Data(), settings_.primaryBeamLimit);
+      writer.SetPolarization(polarization);
+      writer.Write(name.GetPrefix(settings_) + "-" + filename_kind + "-pb.fits",
+                   image.Data());
+    }
+  } else if (settings_.polarizations.size() == 1 || filename_kind == "psf") {
     const PrimaryBeamImageSet beam_images = LoadStokesI(image_name);
     PolarizationEnum pol = *settings_.polarizations.begin();
 
@@ -507,7 +530,7 @@ void PrimaryBeam::MakeOrReuse(const ImageFilename& image_name,
   }
   if (!use_existing_beam) {
     Logger::Info << " == Constructing primary beam ==\n";
-    MakeImage(image_name, entry, image_weights, field_id);
+    MakeImage(image_name, entry, std::move(image_weights), field_id);
   }
 }
 
@@ -537,15 +560,10 @@ void PrimaryBeam::MakeImage(const ImageFilename& image_name,
   std::vector<aocommon::HMC4x4> result;
   double ms_weight_sum = 0;
   for (const MSProviderInfo& ms_provider_info : ms_providers_) {
-    const MSSelection& selection = *ms_provider_info.selection;
-    double central_frequency;
-    {
-      aocommon::BandData band(ms_provider_info.provider->Band(),
-                              selection.ChannelRangeStart(),
-                              selection.ChannelRangeEnd());
-      central_frequency = band.CentreFrequency();
-    }
+    const double central_frequency =
+        ms_provider_info.provider->SelectedBands().CentreFrequency();
 
+    const MSSelection& selection = *ms_provider_info.selection;
     std::vector<aocommon::HMC4x4> ms_beam;
     const double ms_weight =
         MakeBeamForMS(ms_beam, *ms_provider_info.provider, selection,
@@ -590,7 +608,13 @@ double PrimaryBeam::MakeBeamForMS(
       GetBeamIntervals(ms_provider, seconds_before_beam_update_);
   Logger::Debug << "Dividing MS in " << intervals.size() << " intervals.\n";
 
-  SynchronizedMS ms = ms_provider.MS();
+  std::optional<SynchronizedMS> optional_ms = ms_provider.MsIfAvailable();
+  if (!optional_ms) {
+    throw std::runtime_error(
+        "To correct for the beam, an on-disk measurement set is currently "
+        "required");
+  }
+  SynchronizedMS& ms = *optional_ms;
   const everybeam::TelescopeType telescope_type =
       everybeam::GetTelescopeType(*ms);
   std::unique_ptr<everybeam::telescope::Telescope> telescope =
@@ -696,24 +720,23 @@ void PrimaryBeam::CalculateStationWeights(const ImageWeights& imageWeights,
   casacore::MSAntenna antenna_table(ms->antenna());
   aocommon::UVector<double> per_antenna_weights(antenna_table.nrow(), 0.0);
 
-  aocommon::MultiBandData multi_band(ms->spectralWindow(),
-                                     ms->dataDescription());
-  const size_t n_channels =
-      selection.ChannelRangeEnd() - selection.ChannelRangeStart();
+  const aocommon::MultiBandData& multi_band =
+      ms_reader.Provider().SelectedBands();
   const size_t n_polarizations = ms_reader.NPolarizations();
-  aocommon::UVector<float> weight_array(n_channels * n_polarizations);
-  const aocommon::BandData band = multi_band[ms_reader.DataDescId()];
+  aocommon::UVector<float> weight_array(multi_band.MaxBandChannels() *
+                                        n_polarizations);
   while (ms_reader.CurrentRowAvailable() && current_row <= end_row) {
-    MSProvider::MetaData meta_data;
-    ms_reader.ReadMeta(meta_data);
+    MSProvider::MetaData metadata;
+    ms_reader.ReadMeta(metadata);
     ms_reader.ReadWeights(weight_array.data());
+    const aocommon::BandData band = multi_band[metadata.data_desc_id];
 
-    for (size_t ch = 0; ch != n_channels; ++ch) {
-      const double u = meta_data.uInM / band.ChannelWavelength(ch);
-      const double v = meta_data.vInM / band.ChannelWavelength(ch);
+    for (size_t ch = 0; ch != band.ChannelCount(); ++ch) {
+      const double u = metadata.u_in_m / band.ChannelWavelength(ch);
+      const double v = metadata.v_in_m / band.ChannelWavelength(ch);
       const double iw = imageWeights.GetWeight(u, v);
       const double w = weight_array[ch * n_polarizations] * iw;
-      baselineWeights.Value(meta_data.antenna1, meta_data.antenna2) += w;
+      baselineWeights.Value(metadata.antenna1, metadata.antenna2) += w;
     }
     ms_reader.NextInputRow();
     ++current_row;
